# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_ozbay.

This module contains unit tests for abydos.distance.Ozbay
"""

import unittest

from abydos.distance import Ozbay


class OzbayTestCases(unittest.TestCase):
    """Test Ozbay metric functions.

    abydos.distance.Ozbay
    """

    cmp = Ozbay()

    def test_ozbay_dist_abs(self):
        """Test abydos.distance.Ozbay.dist_abs."""
        self.assertEqual(self.cmp.dist_abs('', ''), 0.0)

        self.assertAlmostEqual(
            self.cmp.dist_abs('piccadilly', 'bandage'), 73.63636363636363
        )
        self.assertAlmostEqual(self.cmp.dist_abs('abcd', 'efgh'), 16)

        # Test cases from https://github.com/hakanozbay/ozbay-metric
        self.assertEqual(self.cmp.dist_abs('ban', 'ban'), 0.0)
        self.assertAlmostEqual(self.cmp.dist_abs('ban', 'bane'), 0.3333333333)
        self.assertAlmostEqual(self.cmp.dist_abs('ban', 'band'), 0.3333333333)
        self.assertEqual(self.cmp.dist_abs('ban', 'bat'), 0.75)
        self.assertAlmostEqual(self.cmp.dist_abs('ban', 'bands'), 1.3333333333)
        self.assertEqual(self.cmp.dist_abs('ban', 'banana'), 2.0)
        self.assertAlmostEqual(
            self.cmp.dist_abs('ban', 'bandana'), 2.3333333333
        )
        self.assertEqual(self.cmp.dist_abs('ban', 'bandit'), 3.0)
        self.assertAlmostEqual(
            self.cmp.dist_abs('ban', 'bandage'), 4.6666666666
        )

        self.assertEqual(self.cmp.dist_abs('piccadilly', 'piccadilly'), 0.0)
        self.assertEqual(self.cmp.dist_abs('piccadilly', 'piccadilyl'), 0.25)
        self.assertAlmostEqual(
            self.cmp.dist_abs('piccadilly', 'piccadlily'), 0.3333333333
        )
        self.assertEqual(self.cmp.dist_abs('piccadilly', 'picacdilly'), 0.4)
        self.assertEqual(self.cmp.dist_abs('piccadilly', 'picadily'), 0.4)
        self.assertEqual(self.cmp.dist_abs('picadily', 'piccadilly'), 0.5)
        self.assertAlmostEqual(
            self.cmp.dist_abs('piccadilly', 'picacdlily'), 1.3333333333
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs('ipcacdily', 'piccadilly'), 1.4814814814814814
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs('piccadilly', 'ipcacdily'), 1.333333333
        )
        self.assertEqual(self.cmp.dist_abs('piccadilly', 'pcicadlyil'), 2.0)

    def test_ozbay_dist(self):
        """Test abydos.distance.Ozbay.dist."""
        self.assertEqual(self.cmp.dist('', ''), 0)

        self.assertAlmostEqual(
            self.cmp.dist('piccadilly', 'bandage'), 0.9467532467532467
        )
        self.assertAlmostEqual(self.cmp.dist('abcd', 'efgh'), 1.0)

        # Test cases from https://github.com/hakanozbay/ozbay-metric
        self.assertEqual(self.cmp.dist('ban', 'ban'), 0.0)
        self.assertAlmostEqual(
            self.cmp.dist('ban', 'bane'), 0.006944444444444444
        )
        self.assertAlmostEqual(
            self.cmp.dist('ban', 'band'), 0.006944444444444444
        )
        self.assertEqual(self.cmp.dist('ban', 'bat'), 0.02777777777777778)
        self.assertAlmostEqual(
            self.cmp.dist('ban', 'bands'), 0.03555555555555556
        )
        self.assertEqual(self.cmp.dist('ban', 'banana'), 0.05555555555555555)
        self.assertAlmostEqual(
            self.cmp.dist('ban', 'bandana'), 0.0634920634920635
        )
        self.assertEqual(self.cmp.dist('ban', 'bandit'), 0.08333333333333333)
        self.assertAlmostEqual(
            self.cmp.dist('ban', 'bandage'), 0.126984126984127
        )

        self.assertEqual(self.cmp.dist('piccadilly', 'piccadilly'), 0.0)
        self.assertEqual(
            self.cmp.dist('piccadilly', 'piccadilyl'), 0.0004999999999999999
        )
        self.assertAlmostEqual(
            self.cmp.dist('piccadilly', 'piccadlily'), 0.0013333333333333335
        )
        self.assertEqual(self.cmp.dist('piccadilly', 'picacdilly'), 0.002)
        self.assertEqual(self.cmp.dist('piccadilly', 'picadily'), 0.0025)
        self.assertEqual(self.cmp.dist('picadily', 'piccadilly'), 0.003125)
        self.assertAlmostEqual(
            self.cmp.dist('piccadilly', 'picacdlily'), 0.009333333333333334
        )
        self.assertAlmostEqual(
            self.cmp.dist('ipcacdily', 'piccadilly'), 0.011522633744855966
        )
        self.assertAlmostEqual(
            self.cmp.dist('piccadilly', 'ipcacdily'), 0.01037037037037037
        )
        self.assertEqual(self.cmp.dist('piccadilly', 'pcicadlyil'), 0.014)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_damerau_levenshtein.

This module contains unit tests for abydos.distance.DamerauLevenshtein
"""

import unittest

from abydos.distance import DamerauLevenshtein


class DamerauLevenshteinTestCases(unittest.TestCase):
    """Test Damerau-Levenshtein functions.

    abydos.distance.DamerauLevenshtein
    """

    cmp = DamerauLevenshtein()
    cmp571010 = DamerauLevenshtein(cost=(5, 7, 10, 10))
    cmp1010510 = DamerauLevenshtein(cost=(10, 10, 5, 10))
    cmp55105 = DamerauLevenshtein(cost=(5, 5, 10, 5))
    cmp1010105 = DamerauLevenshtein(cost=(10, 10, 10, 5))

    def test_damerau_levenshtein_dist_abs(self):
        """Test abydos.distance.DamerauLevenshtein.dist_abs."""
        self.assertEqual(self.cmp.dist_abs('', ''), 0)
        self.assertEqual(self.cmp.dist_abs('CA', 'CA'), 0)
        self.assertEqual(self.cmp.dist_abs('CA', 'ABC'), 2)
        self.assertEqual(self.cmp571010.dist_abs('', 'b'), 5)
        self.assertEqual(self.cmp571010.dist_abs('a', 'ab'), 5)
        self.assertEqual(self.cmp571010.dist_abs('b', ''), 7)
        self.assertEqual(self.cmp571010.dist_abs('ab', 'a'), 7)
        self.assertEqual(self.cmp1010510.dist_abs('a', 'b'), 5)
        self.assertEqual(self.cmp1010510.dist_abs('ac', 'bc'), 5)
        self.assertEqual(self.cmp55105.dist_abs('ab', 'ba'), 5)
        self.assertEqual(self.cmp55105.dist_abs('abc', 'bac'), 5)
        self.assertEqual(self.cmp55105.dist_abs('cab', 'cba'), 5)
        self.assertRaises(ValueError, self.cmp1010105.dist_abs, 'ab', 'ba')

    def test_damerau_dist(self):
        """Test abydos.distance.DamerauLevenshtein.dist."""
        self.assertEqual(self.cmp.dist('', ''), 0)

        self.assertEqual(self.cmp.dist('a', 'a'), 0)
        self.assertEqual(self.cmp.dist('ab', 'ab'), 0)
        self.assertEqual(self.cmp.dist('', 'a'), 1)
        self.assertEqual(self.cmp.dist('', 'ab'), 1)
        self.assertEqual(self.cmp.dist('a', 'c'), 1)

        self.assertAlmostEqual(self.cmp.dist('abc', 'ac'), 1 / 3)
        self.assertAlmostEqual(self.cmp.dist('abbc', 'ac'), 1 / 2)
        self.assertAlmostEqual(self.cmp.dist('abbc', 'abc'), 1 / 4)

        self.assertAlmostEqual(self.cmp.dist('CA', 'ABC'), 2 / 3)
        self.assertAlmostEqual(self.cmp571010.dist('', 'b'), 1)
        self.assertAlmostEqual(self.cmp571010.dist('a', 'ab'), 1 / 2)
        self.assertAlmostEqual(self.cmp571010.dist('b', ''), 1)
        self.assertAlmostEqual(self.cmp571010.dist('ab', 'a'), 1 / 2)
        self.assertAlmostEqual(self.cmp1010510.dist('a', 'b'), 1 / 2)
        self.assertAlmostEqual(self.cmp1010510.dist('ac', 'bc'), 1 / 4)
        self.assertAlmostEqual(self.cmp55105.dist('ab', 'ba'), 1 / 2)
        self.assertAlmostEqual(self.cmp55105.dist('abc', 'bac'), 1 / 3)
        self.assertAlmostEqual(self.cmp55105.dist('cab', 'cba'), 1 / 3)
        self.assertRaises(ValueError, self.cmp1010105.dist, 'ab', 'ba')

    def test_damerau_sim(self):
        """Test abydos.distance.DamerauLevenshtein.sim."""
        self.assertEqual(self.cmp.sim('', ''), 1)

        self.assertEqual(self.cmp.sim('a', 'a'), 1)
        self.assertEqual(self.cmp.sim('ab', 'ab'), 1)
        self.assertEqual(self.cmp.sim('', 'a'), 0)
        self.assertEqual(self.cmp.sim('', 'ab'), 0)
        self.assertEqual(self.cmp.sim('a', 'c'), 0)

        self.assertAlmostEqual(self.cmp.sim('abc', 'ac'), 2 / 3)
        self.assertAlmostEqual(self.cmp.sim('abbc', 'ac'), 1 / 2)
        self.assertAlmostEqual(self.cmp.sim('abbc', 'abc'), 3 / 4)

        self.assertAlmostEqual(self.cmp.sim('CA', 'ABC'), 1 / 3)
        self.assertAlmostEqual(self.cmp571010.sim('', 'b'), 0)
        self.assertAlmostEqual(self.cmp571010.sim('a', 'ab'), 1 / 2)
        self.assertAlmostEqual(self.cmp571010.sim('b', ''), 0)
        self.assertAlmostEqual(self.cmp571010.sim('ab', 'a'), 1 / 2)
        self.assertAlmostEqual(self.cmp1010510.sim('a', 'b'), 1 / 2)
        self.assertAlmostEqual(self.cmp1010510.sim('ac', 'bc'), 3 / 4)
        self.assertAlmostEqual(self.cmp55105.sim('ab', 'ba'), 1 / 2)
        self.assertAlmostEqual(self.cmp55105.sim('abc', 'bac'), 2 / 3)
        self.assertAlmostEqual(self.cmp55105.sim('cab', 'cba'), 2 / 3)
        self.assertRaises(ValueError, self.cmp1010105.sim, 'ab', 'ba')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_fuzzy_soundex.

This module contains unit tests for abydos.phonetic.FuzzySoundex
"""

import unittest

from abydos.phonetic import FuzzySoundex


class FuzzySoundexTestCases(unittest.TestCase):
    """Test Fuzzy Soundex functions.

    test cases for abydos.phonetic.FuzzySoundex
    """

    pa = FuzzySoundex()
    pa_4 = FuzzySoundex(4)

    def test_fuzzy_soundex(self):
        """Test abydos.phonetic.FuzzySoundex."""
        self.assertEqual(self.pa.encode(''), '00000')
        # http://wayback.archive.org/web/20100629121128/http://www.ir.iit.edu/publications/downloads/IEEESoundexV5.pdf
        self.assertEqual(self.pa.encode('Kristen'), 'K6935')
        self.assertEqual(self.pa.encode('Krissy'), 'K6900')
        self.assertEqual(self.pa.encode('Christen'), 'K6935')

        # http://books.google.com/books?id=LZrT6eWf9NMC&lpg=PA76&ots=Tex3FqNwGP&dq=%22phonix%20algorithm%22&pg=PA75#v=onepage&q=%22phonix%20algorithm%22&f=false
        self.assertEqual(self.pa_4.encode('peter'), 'P360')
        self.assertEqual(self.pa_4.encode('pete'), 'P300')
        self.assertEqual(self.pa_4.encode('pedro'), 'P360')
        self.assertEqual(self.pa_4.encode('stephen'), 'S315')
        self.assertEqual(self.pa_4.encode('steve'), 'S310')
        self.assertEqual(self.pa_4.encode('smith'), 'S530')
        self.assertEqual(self.pa_4.encode('smythe'), 'S530')
        self.assertEqual(self.pa_4.encode('gail'), 'G400')
        self.assertEqual(self.pa_4.encode('gayle'), 'G400')
        self.assertEqual(self.pa_4.encode('christine'), 'K693')
        self.assertEqual(self.pa_4.encode('christina'), 'K693')
        self.assertEqual(self.pa_4.encode('kristina'), 'K693')

        # etc. (for code coverage)
        self.assertEqual(self.pa.encode('Wight'), 'W3000')
        self.assertEqual(self.pa.encode('Hardt'), 'H6000')
        self.assertEqual(self.pa.encode('Knight'), 'N3000')
        self.assertEqual(self.pa.encode('Czech'), 'S7000')
        self.assertEqual(self.pa.encode('Tsech'), 'S7000')
        self.assertEqual(self.pa.encode('gnomic'), 'N5900')
        self.assertEqual(self.pa.encode('Wright'), 'R3000')
        self.assertEqual(self.pa.encode('Hrothgar'), 'R3760')
        self.assertEqual(self.pa.encode('Hwaet'), 'W3000')
        self.assertEqual(self.pa.encode('Grant'), 'G6300')
        self.assertEqual(self.pa.encode('Hart'), 'H6000')
        self.assertEqual(self.pa.encode('Hardt'), 'H6000')

        # max_length bounds tests
        self.assertEqual(
            FuzzySoundex(max_length=-1).encode('Niall'),
            'N400000000000000000000000000000000000000000000000000000000000000',
        )
        self.assertEqual(FuzzySoundex(max_length=0).encode('Niall'), 'N400')

        # zero_pad tests
        self.assertEqual(
            FuzzySoundex(max_length=-1, zero_pad=False).encode('Niall'), 'N4'
        )
        self.assertEqual(
            FuzzySoundex(max_length=0, zero_pad=False).encode('Niall'), 'N4'
        )
        self.assertEqual(
            FuzzySoundex(max_length=0, zero_pad=True).encode('Niall'), 'N400'
        )
        self.assertEqual(
            FuzzySoundex(max_length=4, zero_pad=False).encode(''), '0'
        )
        self.assertEqual(
            FuzzySoundex(max_length=4, zero_pad=True).encode(''), '0000'
        )

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('pete'), 'PT')
        self.assertEqual(self.pa.encode_alpha('pedro'), 'PTR')
        self.assertEqual(self.pa.encode_alpha('stephen'), 'STPN')
        self.assertEqual(self.pa.encode_alpha('steve'), 'STP')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stemmer.test_stemmer_porter2.

This module contains unit tests for abydos.stemmer.Porter2
"""

import unittest

from abydos.stemmer import Porter2


from .. import _corpus_file


class Porter2TestCases(unittest.TestCase):
    """Test Porter2 functions.

    abydos.stemmer.Porter2
    """

    stmr = Porter2()
    stmr._vowels = set('aeiouy')  # noqa: SF01
    stmr_ee = Porter2(early_english=True)
    stmr_ee._vowels = set('aeiouy')  # noqa: SF01

    def test_porter2(self):
        """Test abydos.stemmer.Porter2."""
        # base case
        self.assertEqual(self.stmr.stem(''), '')

        # simple cases
        self.assertEqual(self.stmr.stem('c'), 'c')
        self.assertEqual(self.stmr.stem('da'), 'da')
        self.assertEqual(self.stmr.stem('ad'), 'ad')
        self.assertEqual(self.stmr.stem('sing'), 'sing')
        self.assertEqual(self.stmr.stem('singing'), 'sing')

        # missed branch test cases
        self.assertEqual(self.stmr.stem('capitalism'), 'capit')
        self.assertEqual(self.stmr.stem('fatalism'), 'fatal')
        self.assertEqual(self.stmr.stem("dog's"), 'dog')
        self.assertEqual(self.stmr.stem("A's'"), 'a')
        self.assertEqual(self.stmr.stem('agreedly'), 'agre')
        self.assertEqual(self.stmr.stem('feedly'), 'feed')
        self.assertEqual(self.stmr.stem('stional'), 'stional')
        self.assertEqual(self.stmr.stem('palism'), 'palism')
        self.assertEqual(self.stmr.stem('sization'), 'sizat')
        self.assertEqual(self.stmr.stem('licated'), 'licat')
        self.assertEqual(self.stmr.stem('lical'), 'lical')
        self.assertEqual(self.stmr.stem('clessly'), 'clessli')
        self.assertEqual(self.stmr.stem('tably'), 'tabli')
        self.assertEqual(self.stmr.stem('sizer'), 'sizer')
        self.assertEqual(self.stmr.stem('livity'), 'liviti')

    def test_porter2_early_english(self):
        """Test abydos.stemmer.Porter2 (early English)."""
        # base case
        self.assertEqual(self.stmr_ee.stem(''), '')

        # simple cases (no different from regular stemmer)
        self.assertEqual(self.stmr_ee.stem('c'), 'c')
        self.assertEqual(self.stmr_ee.stem('da'), 'da')
        self.assertEqual(self.stmr_ee.stem('ad'), 'ad')
        self.assertEqual(self.stmr_ee.stem('sing'), 'sing')
        self.assertEqual(self.stmr_ee.stem('singing'), 'sing')

        # make
        self.assertEqual(self.stmr_ee.stem('make'), 'make')
        self.assertEqual(self.stmr_ee.stem('makes'), 'make')
        self.assertEqual(self.stmr_ee.stem('maketh'), 'make')
        self.assertEqual(self.stmr_ee.stem('makest'), 'make')

        # say
        self.assertEqual(self.stmr_ee.stem('say'), 'say')
        self.assertEqual(self.stmr_ee.stem('says'), 'say')
        self.assertEqual(self.stmr_ee.stem('sayeth'), 'say')
        self.assertEqual(self.stmr_ee.stem('sayest'), 'say')

        # missed branch test cases
        self.assertEqual(self.stmr_ee.stem('best'), 'best')
        self.assertEqual(self.stmr_ee.stem('meth'), 'meth')

    def test_porter2_snowball(self):
        """Test abydos.stemmer.Porter2 (Snowball testset).

        These test cases are from
        http://snowball.tartarus.org/algorithms/english/diffs.txt
        """
        #  Snowball Porter test set
        with open(_corpus_file('snowball_porter2.csv')) as snowball_ts:
            next(snowball_ts)
            for line in snowball_ts:
                if line[0] != '#':
                    line = line.strip().split(',')
                    word, stem = line[0], line[1]
                    self.assertEqual(self.stmr.stem(word), stem.lower())


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_spanish_metaphone.

This module contains unit tests for abydos.phonetic.SpanishMetaphone
"""

import unittest

from abydos.phonetic import SpanishMetaphone


class SpanishMetaphoneTestCases(unittest.TestCase):
    """Test Spanish Metaphone functions.

    test cases for abydos.phonetic.SpanishMetaphone
    """

    pa = SpanishMetaphone()
    pa_mod = SpanishMetaphone(modified=True)

    def test_spanish_metaphone(self):
        """Test abydos.phonetic.SpanishMetaphone."""
        # Base case
        self.assertEqual(self.pa.encode(''), '')

        # Examples given in
        # https://github.com/amsqr/Spanish-Metaphone/blob/master/phonetic_algorithms_es.py
        self.assertEqual(self.pa.encode('X'), 'X')
        self.assertEqual(self.pa.encode('xplosion'), 'EXPLSN')
        self.assertEqual(self.pa.encode('escalera'), 'ESKLR')
        self.assertEqual(self.pa.encode('scalera'), 'ESKLR')
        self.assertEqual(self.pa.encode('mi'), 'M')
        self.assertEqual(self.pa.encode('tu'), 'T')
        self.assertEqual(self.pa.encode('su'), 'S')
        self.assertEqual(self.pa.encode('te'), 'T')
        self.assertEqual(self.pa.encode('ochooomiiiillllllll'), 'OXMYY')
        self.assertEqual(self.pa.encode('complicado'), 'KMPLKD')
        self.assertEqual(self.pa.encode('ácaro'), 'AKR')
        self.assertEqual(self.pa.encode('ácido'), 'AZD')
        self.assertEqual(self.pa.encode('clown'), 'KLUN')
        self.assertEqual(self.pa.encode('down'), 'DUN')
        self.assertEqual(self.pa.encode('col'), 'KL')
        self.assertEqual(self.pa.encode('clon'), 'KLN')
        self.assertEqual(self.pa.encode('waterpolo'), 'UTRPL')
        self.assertEqual(self.pa.encode('aquino'), 'AKN')
        self.assertEqual(self.pa.encode('rebosar'), 'RVSR')
        self.assertEqual(self.pa.encode('rebozar'), 'RVZR')
        self.assertEqual(self.pa.encode('grajea'), 'GRJ')
        self.assertEqual(self.pa.encode('gragea'), 'GRJ')
        self.assertEqual(self.pa.encode('encima'), 'ENZM')
        self.assertEqual(self.pa.encode('enzima'), 'ENZM')
        self.assertEqual(self.pa.encode('alhamar'), 'ALAMR')
        self.assertEqual(self.pa.encode('abollar'), 'AVYR')
        self.assertEqual(self.pa.encode('aboyar'), 'AVYR')
        self.assertEqual(self.pa.encode('huevo'), 'UV')
        self.assertEqual(self.pa.encode('webo'), 'UV')
        self.assertEqual(self.pa.encode('macho'), 'MX')
        self.assertEqual(self.pa.encode('xocolate'), 'XKLT')
        self.assertEqual(self.pa.encode('chocolate'), 'XKLT')
        self.assertEqual(self.pa.encode('axioma'), 'AXM')
        self.assertEqual(self.pa.encode('abedul'), 'AVDL')
        self.assertEqual(self.pa.encode('a'), 'A')
        self.assertEqual(self.pa.encode('gengibre'), 'JNJVR')
        self.assertEqual(self.pa.encode('yema'), 'YM')
        self.assertEqual(self.pa.encode('wHISKY'), 'UISKY')
        self.assertEqual(self.pa.encode('google'), 'GGL')
        self.assertEqual(self.pa.encode('xilófono'), 'XLFN')
        self.assertEqual(self.pa.encode('web'), 'UV')
        self.assertEqual(self.pa.encode('guerra'), 'GRR')
        self.assertEqual(self.pa.encode('pingüino'), 'PNUN')
        self.assertEqual(self.pa.encode('si'), 'S')
        self.assertEqual(self.pa.encode('ke'), 'K')
        self.assertEqual(self.pa.encode('que'), 'K')
        self.assertEqual(self.pa.encode('tu'), 'T')
        self.assertEqual(self.pa.encode('gato'), 'GT')
        self.assertEqual(self.pa.encode('gitano'), 'JTN')
        self.assertEqual(self.pa.encode('queso'), 'KS')
        self.assertEqual(self.pa.encode('paquete'), 'PKT')
        self.assertEqual(self.pa.encode('cuco'), 'KK')
        self.assertEqual(self.pa.encode('perro'), 'PRR')
        self.assertEqual(self.pa.encode('pero'), 'PR')
        self.assertEqual(self.pa.encode('arrebato'), 'ARRVT')
        self.assertEqual(self.pa.encode('hola'), 'OL')
        self.assertEqual(self.pa.encode('zapato'), 'ZPT')
        self.assertEqual(self.pa.encode('españa'), 'ESPNY')
        self.assertEqual(self.pa.encode('garrulo'), 'GRRL')
        self.assertEqual(self.pa.encode('expansión'), 'EXPNSN')
        self.assertEqual(self.pa.encode('membrillo'), 'MMVRY')
        self.assertEqual(self.pa.encode('jamón'), 'JMN')
        self.assertEqual(self.pa.encode('risa'), 'RS')
        self.assertEqual(self.pa.encode('caricia'), 'KRZ')
        self.assertEqual(self.pa.encode('llaves'), 'YVS')
        self.assertEqual(self.pa.encode('paella'), 'PY')
        self.assertEqual(self.pa.encode('cerilla'), 'ZRY')

        # tests from file:///home/chrislit/Downloads/ICTRS_2016_12.pdf
        # including of the modified version of the algorithm
        self.assertEqual(self.pa.encode('Caricia'), 'KRZ')
        self.assertEqual(self.pa_mod.encode('Caricia'), 'KRZ')
        self.assertEqual(self.pa.encode('Llaves'), 'YVS')
        self.assertEqual(self.pa_mod.encode('Llaves'), 'YVZ')
        self.assertEqual(self.pa.encode('Paella'), 'PY')
        self.assertEqual(self.pa_mod.encode('Paella'), 'PY')
        self.assertEqual(self.pa.encode('Cerilla'), 'ZRY')
        self.assertEqual(self.pa_mod.encode('Cerilla'), 'ZRY')
        self.assertEqual(self.pa.encode('Empeorar'), 'EMPRR')
        self.assertEqual(self.pa_mod.encode('Empeorar'), 'ENPRR')
        self.assertEqual(self.pa.encode('Embotellar'), 'EMVTYR')
        self.assertEqual(self.pa_mod.encode('Embotellar'), 'ENVTYR')
        self.assertEqual(self.pa.encode('Hoy'), 'OY')
        self.assertEqual(self.pa_mod.encode('Hoy'), 'OY')
        self.assertEqual(self.pa.encode('Xochimilco'), 'XXMLK')
        self.assertEqual(self.pa_mod.encode('Xochimilco'), 'XXMLK')
        self.assertEqual(self.pa.encode('Psiquiatra'), 'PSKTR')
        self.assertEqual(self.pa_mod.encode('Psiquiatra'), 'ZKTR')
        self.assertEqual(self.pa.encode('siquiatra'), 'SKTR')
        self.assertEqual(self.pa_mod.encode('siquiatra'), 'ZKTR')
        self.assertEqual(self.pa.encode('Obscuro'), 'OVSKR')
        self.assertEqual(self.pa_mod.encode('Obscuro'), 'OZKR')
        self.assertEqual(self.pa.encode('Oscuro'), 'OSKR')
        self.assertEqual(self.pa_mod.encode('Oscuro'), 'OZKR')
        self.assertEqual(self.pa.encode('Combate'), 'KMVT')
        self.assertEqual(self.pa_mod.encode('Combate'), 'KNVT')
        self.assertEqual(self.pa.encode('Convate'), 'KNVT')
        self.assertEqual(self.pa_mod.encode('Convate'), 'KNVT')
        self.assertEqual(self.pa.encode('Conbate'), 'KNVT')
        self.assertEqual(self.pa_mod.encode('Conbate'), 'KNVT')
        self.assertEqual(self.pa.encode('Comportar'), 'KMPRTR')
        self.assertEqual(self.pa_mod.encode('Comportar'), 'KNPRTR')
        self.assertEqual(self.pa.encode('Conportar'), 'KNPRTR')
        self.assertEqual(self.pa_mod.encode('Conportar'), 'KNPRTR')
        self.assertEqual(self.pa.encode('Zapato'), 'ZPT')
        self.assertEqual(self.pa_mod.encode('Zapato'), 'ZPT')
        self.assertEqual(self.pa.encode('Sapato'), 'SPT')
        self.assertEqual(self.pa_mod.encode('Sapato'), 'ZPT')
        self.assertEqual(self.pa.encode('Escalera'), 'ESKLR')
        self.assertEqual(self.pa_mod.encode('Escalera'), 'EZKLR')
        self.assertEqual(self.pa.encode('scalera'), 'ESKLR')
        self.assertEqual(self.pa_mod.encode('scalera'), 'EZKLR')

        # terms from algorithm/source
        self.assertEqual(self.pa.encode('acción'), 'AXN')
        self.assertEqual(self.pa.encode('reacción'), 'RXN')
        self.assertEqual(self.pa.encode('cesar'), 'ZSR')
        self.assertEqual(self.pa.encode('cien'), 'ZN')
        self.assertEqual(self.pa.encode('cid'), 'ZD')
        self.assertEqual(self.pa.encode('conciencia'), 'KNZNZ')
        self.assertEqual(self.pa.encode('gente'), 'JNT')
        self.assertEqual(self.pa.encode('ecologia'), 'EKLJ')

        # completing coverage
        self.assertEqual(self.pa.encode('hola'), 'OL')
        self.assertEqual(self.pa.encode('aqi'), 'AK')
        self.assertEqual(self.pa.encode('hjordis'), 'HJRDS')


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_typo.

This module contains unit tests for abydos.distance.Typo
"""

import unittest

from abydos.distance import Typo


class TypoTestCases(unittest.TestCase):
    """Test Typo functions.

    abydos.distance.Typo
    """

    cmp = Typo()
    cmp_auto = Typo(layout='auto', failsafe=True)

    def test_typo_dist_abs(self):
        """Test abydos.distance.Typo.dist_abs."""
        # Base cases
        self.assertEqual(self.cmp.dist_abs('', ''), 0)
        self.assertEqual(self.cmp.dist_abs('', 'typo'), 4)
        self.assertEqual(self.cmp.dist_abs('typo', ''), 4)

        self.assertEqual(self.cmp.dist_abs('asdf', 'zxcv'), 2)
        self.assertEqual(self.cmp.dist_abs('asdf', 'ASDF'), 1)
        self.assertEqual(self.cmp.dist_abs('asdf', 'qsdf'), 0.5)

        self.assertAlmostEqual(
            Typo(metric='euclidean').dist_abs('asdf', 'asdt'), 0.70710677
        )
        self.assertAlmostEqual(
            Typo(metric='manhattan').dist_abs('asdf', 'asdt'), 1
        )
        self.assertAlmostEqual(
            Typo(metric='log-euclidean').dist_abs('asdf', 'asdt'), 0.4406868
        )
        self.assertAlmostEqual(
            Typo(metric='log-manhattan').dist_abs('asdf', 'asdt'), 0.54930615
        )

        self.assertEqual(self.cmp_auto.dist_abs('Schluß', 'Schluss'), 3)
        self.assertAlmostEqual(
            self.cmp_auto.dist_abs('délicat', 'delicate'), 1.7071068
        )
        self.assertEqual(self.cmp_auto.dist_abs('비빔밥', 'Bibimbap'), 11)

        self.assertRaises(ValueError, self.cmp.dist_abs, 'asdf', 'Ösdf')

    def test_typo_sim(self):
        """Test abydos.distance.Typo.sim."""
        # Base cases
        self.assertEqual(self.cmp.sim('', ''), 1)
        self.assertEqual(self.cmp.sim('', 'typo'), 0)
        self.assertEqual(self.cmp.sim('typo', ''), 0)

        self.assertEqual(self.cmp.sim('asdf', 'zxcv'), 0.5)
        self.assertEqual(self.cmp.sim('asdf', 'ASDF'), 0.75)
        self.assertEqual(self.cmp.sim('asdf', 'qsdf'), 0.875)

        self.assertAlmostEqual(
            Typo(metric='euclidean').sim('asdf', 'asdt'), 1 - (0.70710677 / 4)
        )
        self.assertAlmostEqual(
            Typo(metric='manhattan').sim('asdf', 'asdt'), 0.75
        )
        self.assertAlmostEqual(
            Typo(metric='log-euclidean').sim('asdf', 'asdt'),
            1 - (0.4406868 / 4),
        )
        self.assertAlmostEqual(
            Typo(metric='log-manhattan').sim('asdf', 'asdt'),
            1 - (0.54930615 / 4),
        )

    def test_typo_dist(self):
        """Test abydos.distance.Typo.dist."""
        # Base cases
        self.assertEqual(self.cmp.dist('', ''), 0)
        self.assertEqual(self.cmp.dist('', 'typo'), 1)
        self.assertEqual(self.cmp.dist('typo', ''), 1)

        self.assertEqual(self.cmp.dist('asdf', 'zxcv'), 0.5)
        self.assertEqual(self.cmp.dist('asdf', 'ASDF'), 0.25)
        self.assertEqual(self.cmp.dist('asdf', 'qsdf'), 0.125)

        self.assertAlmostEqual(
            Typo(metric='euclidean').dist('asdf', 'asdt'), 0.70710677 / 4
        )
        self.assertAlmostEqual(
            Typo(metric='manhattan').dist('asdf', 'asdt'), 0.25
        )
        self.assertAlmostEqual(
            Typo(metric='log-euclidean').dist('asdf', 'asdt'), 0.4406868 / 4
        )
        self.assertAlmostEqual(
            Typo(metric='log-manhattan').dist('asdf', 'asdt'), 0.54930615 / 4
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_fuzzywuzzy_partial_string.

This module contains unit tests for abydos.distance.FuzzyWuzzyPartialString
"""

import unittest

from abydos.distance import FuzzyWuzzyPartialString


class FuzzyWuzzyPartialStringTestCases(unittest.TestCase):
    """Test FuzzyWuzzyPartialString functions.

    abydos.distance.FuzzyWuzzyPartialString
    """

    cmp = FuzzyWuzzyPartialString()

    def test_fuzzywuzzy_partial_string_sim(self):
        """Test abydos.distance.FuzzyWuzzyPartialString.sim."""
        # Base cases
        self.assertEqual(self.cmp.sim('', ''), 1.0)
        self.assertEqual(self.cmp.sim('a', ''), 1.0)
        self.assertEqual(self.cmp.sim('', 'a'), 1.0)
        self.assertEqual(self.cmp.sim('abc', ''), 1.0)
        self.assertEqual(self.cmp.sim('', 'abc'), 1.0)
        self.assertEqual(self.cmp.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp.sim('abcd', 'efgh'), 0.0)

        self.assertAlmostEqual(self.cmp.sim('Nigel', 'Niall'), 0.6)
        self.assertAlmostEqual(self.cmp.sim('Niall', 'Nigel'), 0.6)
        self.assertAlmostEqual(self.cmp.sim('Colin', 'Coiln'), 0.8)
        self.assertAlmostEqual(self.cmp.sim('Coiln', 'Colin'), 0.8)
        self.assertAlmostEqual(
            self.cmp.sim('ATCAACGAGT', 'AACGATTAG'), 0.6666666667
        )

        # tests from blog
        self.assertAlmostEqual(
            self.cmp.sim('YANKEES', 'NEW YORK YANKEES'), 1.0
        )
        self.assertAlmostEqual(
            self.cmp.sim('NEW YORK METS', 'NEW YORK YANKEES'),
            0.6923076923076923,
        )
        self.assertAlmostEqual(
            self.cmp.sim(
                'New York Mets vs Atlanta Braves',
                'Atlanta Braves vs New York Mets',
            ),
            0.45161290322580644,
        )

    def test_fuzzywuzzy_partial_string_dist(self):
        """Test abydos.distance.FuzzyWuzzyPartialString.dist."""
        # Base cases
        self.assertEqual(self.cmp.dist('', ''), 0.0)
        self.assertEqual(self.cmp.dist('a', ''), 0.0)
        self.assertEqual(self.cmp.dist('', 'a'), 0.0)
        self.assertEqual(self.cmp.dist('abc', ''), 0.0)
        self.assertEqual(self.cmp.dist('', 'abc'), 0.0)
        self.assertEqual(self.cmp.dist('abc', 'abc'), 0.0)
        self.assertEqual(self.cmp.dist('abcd', 'efgh'), 1.0)

        self.assertAlmostEqual(self.cmp.dist('Nigel', 'Niall'), 0.4)
        self.assertAlmostEqual(self.cmp.dist('Niall', 'Nigel'), 0.4)
        self.assertAlmostEqual(self.cmp.dist('Colin', 'Coiln'), 0.2)
        self.assertAlmostEqual(self.cmp.dist('Coiln', 'Colin'), 0.2)
        self.assertAlmostEqual(
            self.cmp.dist('ATCAACGAGT', 'AACGATTAG'), 0.3333333333
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.util.test_ncr.

This module contains unit tests for abydos.util._ncr
"""

import unittest

from abydos.util._ncr import _ncr


class ProdTestCases(unittest.TestCase):
    """Test cases for abydos.util._ncr."""

    def test_ncr(self):
        """Test abydos.util._ncr."""
        self.assertEqual(_ncr(1, 0), 1)
        self.assertEqual(_ncr(5, 0), 1)

        self.assertEqual(_ncr(1, 2), 0)
        self.assertEqual(_ncr(1, 2), 0)

        self.assertEqual(_ncr(2, 2), 1)
        self.assertEqual(_ncr(10, 10), 1)

        self.assertEqual(_ncr(7, 2), 21)
        self.assertEqual(_ncr(7, 3), 35)
        self.assertEqual(_ncr(4, 3), 4)
        self.assertEqual(_ncr(5, 3), 10)
        self.assertEqual(_ncr(10, 2), 45)
        self.assertEqual(_ncr(100, 3), 161700)
        self.assertEqual(_ncr(80, 5), 24040016)

        # gamma variant
        self.assertAlmostEqual(_ncr(10, 2.5), 77.8023559942)
        self.assertAlmostEqual(_ncr(0, 2.5), 0.12732395447)
        self.assertAlmostEqual(_ncr(2.5, 2.5), 1)
        self.assertAlmostEqual(_ncr(2.5, 2.1), 1.7043970865)


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.tokenizer.test_tokenizer_c_or_v_cluster.

This module contains unit tests for abydos.tokenizer.COrVClusterTokenizer
"""

import unittest

from abydos.tokenizer import COrVClusterTokenizer


class COrVClusterTokenizerTestCases(unittest.TestCase):
    """Test abydos.tokenizer.COrVClusterTokenizer."""

    def test_c_or_v_cluster_tokenizer(self):
        """Test abydos.tokenizer.COrVClusterTokenizer."""
        self.assertEqual(
            sorted(COrVClusterTokenizer().tokenize('').get_list()), []
        )
        self.assertEqual(
            sorted(COrVClusterTokenizer().tokenize('a').get_list()), ['a']
        )

        tok = COrVClusterTokenizer()

        self.assertEqual(
            sorted(tok.tokenize('nelson').get_list()),
            sorted(['n', 'e', 'ls', 'o', 'n']),
        )
        self.assertEqual(
            sorted(tok.tokenize('neilson').get_list()),
            sorted(['n', 'ei', 'ls', 'o', 'n']),
        )
        self.assertEqual(
            sorted(tok.tokenize('peninsular').get_list()),
            sorted(['p', 'e', 'n', 'i', 'ns', 'u', 'l', 'a', 'r']),
        )
        self.assertEqual(
            sorted(tok.tokenize('spectacular').get_list()),
            sorted(['sp', 'e', 'ct', 'a', 'c', 'u', 'l', 'a', 'r']),
        )
        self.assertEqual(
            sorted(tok.tokenize('sufficiently').get_list()),
            sorted(['s', 'u', 'ff', 'i', 'c', 'ie', 'ntl', 'y']),
        )
        self.assertEqual(
            sorted(tok.tokenize('yachting').get_list()),
            sorted(['ya', 'cht', 'i', 'ng']),
        )
        self.assertEqual(
            sorted(tok.tokenize('caterpillars').get_list()),
            sorted(['c', 'a', 't', 'e', 'rp', 'i', 'll', 'a', 'rs']),
        )
        self.assertEqual(
            sorted(tok.tokenize('Götterdämmerung').get_list()),
            sorted(
                ['G', 'ö', 'tt', 'e', 'rd', 'ä', 'mm', 'e', 'r', 'u', 'ng']
            ),
        )

        tok = COrVClusterTokenizer(consonants='ptkbdgmn', vowels='aeiouwy')
        self.assertEqual(
            sorted(tok.tokenize('#winning #losing').get_list()),
            sorted(['#', 'wi', 'nn', 'i', 'ng', '#', 'losing']),
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.


"""abydos.tests.phonetic.

This module contains unit tests for abydos.phonetic
"""

import unittest


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.fingerprint.test_fingerprint_synoname_toolcode.

This module contains unit tests for abydos.fingerprint.SynonameToolcode
"""

import unittest

from abydos.fingerprint import SynonameToolcode


class SynonameToolcodeTestCases(unittest.TestCase):
    """Test Synoname Toolcode function.

    abydos.fingerprint.SynonameToolcode
    """

    fp = SynonameToolcode()

    def test_synoname_toolcode(self):
        """Test abydos.fingerprint.SynonameToolcode."""
        # Base case
        self.assertEqual(self.fp.fingerprint(''), ',,0000000000$$')

        # from Synoname demo
        self.assertEqual(
            self.fp.fingerprint('angelico', 'fra'),
            'angelico,fra,0000000308$044a$af',
        )
        self.assertEqual(
            self.fp.fingerprint('Aelst', 'Willem van', ''),
            'aelst,willem van,0000001005$143a$awv',
        )
        self.assertEqual(self.fp.fingerprint('Afro'), 'afro,,0000000004$$a')
        self.assertEqual(
            self.fp.fingerprint('Afro', 'Basaldella'),
            'afro,basaldella,0000001004$$ab',
        )
        self.assertEqual(
            self.fp.fingerprint('Albright', 'Ivan'),
            'albright,ivan,0000000408$$ai',
        )
        self.assertEqual(
            self.fp.fingerprint('Antonello da Messina'),
            'antonello da messina,,0000000020$022b$adm',
        )
        self.assertEqual(
            self.fp.fingerprint('Albright', 'Ivan Le Lorraine'),
            'albright,ivan le lorraine,0000001608$067b$ail',
        )
        self.assertEqual(
            self.fp.fingerprint('Bazille', 'Frederic', 'Attributed to'),
            'bazille,frederic,1000000807$$bf',
        )
        self.assertEqual(
            self.fp.fingerprint('Bazille', 'Frederick', 'Attributed to'),
            'bazille,frederick,1000000907$$bf',
        )
        self.assertEqual(
            self.fp.fingerprint('Beerstraaten', 'Jan Abrahamsz.'),
            'beerstraaten,jan abrahamsz.,0200001412$$bja',
        )
        self.assertEqual(
            self.fp.fingerprint('Bonifacio di Pitati'),
            'bonifacio di pitati,,0000000019$035b$bdp',
        )
        self.assertEqual(
            self.fp.fingerprint('Breughel the Younger', 'Jan'),
            'breughel the younger,jan,0020000320$134b$btyj',
        )
        self.assertEqual(
            self.fp.fingerprint('Brown', 'W. W.'),
            'brown,w. w.,0200000505$$bw',
        )
        self.assertEqual(
            self.fp.fingerprint('Brueghel II (the Younger)', 'Jan'),
            'brueghel ii (the younger),jan,0120490325$049b134b$bityj',
        )
        self.assertEqual(
            self.fp.fingerprint(
                'Brueghel II (the Younger)', 'Pieter', 'Workshop of'
            ),
            'brueghel ii (the younger),pieter,3120490625$049b134b$bityp',
        )
        self.assertEqual(
            self.fp.fingerprint('Bugiardini', 'Guiliano di Piero di Simone'),
            'bugiardini,guiliano di piero di simone,0000002710$035b035b$bgdps',
        )
        self.assertEqual(
            self.fp.fingerprint('Caravaggio', '', 'Follower of'),
            'caravaggio,,3000000010$$c',
        )
        self.assertEqual(
            self.fp.fingerprint(
                'Caravaggio', 'Michelangelo Merisi da', 'Follower of'
            ),
            'caravaggio,michelangelo merisi da,3000002210$022a$cmd',
        )
        self.assertEqual(
            self.fp.fingerprint('Oost the Younger', 'Jacob van'),
            'oost the younger,jacob van,0020000916$134b143a$otyjv',
        )

        # additional tests for coverage
        self.assertEqual(
            self.fp.fingerprint('Cato the Elder', '', 'Copy of'),
            'cato the elder,,2010000014$133b$cte',
        )
        self.assertEqual(
            self.fp.fingerprint('Cato, the Elder', normalize=2),
            'cato the elder,,0110000014$133b$cte',
        )
        self.assertEqual(
            self.fp.fingerprint('Cato the Elder', normalize=2),
            'cato the elder,,0010000014$133b$cte',
        )
        self.assertEqual(
            self.fp.fingerprint(
                'Lorem ipsum dolor sit amet, '
                + 'consectetur adipiscing elit, '
                + 'sed do eiusmod tempor '
                + 'incididunt ut labore et dolore '
                + 'magna aliqua. Nulla aliquet '
                + 'porttitor lacus luctus accumsan '
                + 'tortor posuere. Egestas purus '
                + 'viverra accumsan in. Ultrices '
                + 'mi tempus imperdiet nulla '
                + 'malesuada pellentesque elit '
                + 'eget gravida. Proin libero nunc '
                + 'consequat interdum varius sit '
                + 'amet mattis vulputate. Mauris '
                + 'ultrices eros in cursus turpis '
                + 'massa tincidunt dui. Faucibus '
                + 'in ornare quam viverra orci '
                + 'sagittis eu volutpat odio. Enim '
                + 'blandit volutpat maecenas '
                + 'volutpat blandit aliquam etiam. '
                + 'Vel quam elementum pulvinar '
                + 'etiam. Duis ut diam quam nulla '
                + 'porttitor massa id.',
                normalize=1,
            ).split(',')[-1],
            '02000060626$068d$lidsacetumnpvgflo',
        )
        self.assertEqual(
            self.fp.fingerprint('Sainte-Vincent'),
            'sainte-vincent,,0100000014$110c$sv',
        )
        self.assertEqual(
            self.fp.fingerprint('Lorem', 'Sainte-Vincent'),
            'lorem,sainte-vincent,0100001405$068d110b$lsvlo',
        )
        self.assertEqual(
            self.fp.fingerprint('Louis II', 'Jean'),
            'louis ii,jean,0000490408$049b068d$lijlo',
        )
        self.assertEqual(
            self.fp.fingerprint('Louis', 'Jean II', normalize=2),
            'louis ii,jean,0000490705$049a068d$ljilo',
        )
        self.assertEqual(
            self.fp.fingerprint('Louis', 'Jean II ', normalize=2),
            'louis ii,jean,0000490805$049b068d$ljilo',
        )
        self.assertEqual(
            self.fp.fingerprint('Louis', 'Jean II-', normalize=2),
            'louis,jean ii-,0100490805$049b068d$ljilo',
        )
        self.assertEqual(
            self.fp.fingerprint('Louis V.', 'Jean', normalize=2),
            'louis v.,jean,0200000408$068d$lvjlo',
        )
        self.assertEqual(
            self.fp.fingerprint('Louis V.', 'Ste.-Jean Ste.', normalize=2),
            'louis v.,ste.-jean ste.,0200001408$068d127b127X$lvsjlo ste',
        )
        self.assertEqual(
            self.fp.fingerprint('Louis IX', 'Jean III II', normalize=2),
            'louis ix iii ii,jean,0000481108$048b049a056b068d$lijlo',
        )
        self.assertEqual(
            self.fp.fingerprint('Louis IX', 'Jean II III', normalize=2),
            'louis ix iii ii,jean,0000481108$048a049a056b068d$lijlo',
        )
        self.assertEqual(
            self.fp.fingerprint('Louis IX', 'Jean II III', normalize=1),
            'louis ix,jean ii iii,0000481108$048a049a056b068d$lijlo',
        )
        self.assertEqual(
            self.fp.fingerprint('Lorem', 'Sainte-Sainte-Vincent'),
            'lorem,sainte-sainte-vincent,0100002105$068d110b$lsvlo',
        )
        self.assertEqual(
            self.fp.fingerprint('Brueghel II', 'I. Jan', normalize=2),
            'brueghel ii,i. jan,0200000611$$bij',
        )
        self.assertEqual(
            self.fp.fingerprint('Brueghel', 'I. Jan II', normalize=2),
            'brueghel,i. jan ii,0200000908$$bij',
        )
        self.assertEqual(
            self.fp.fingerprint('Lorem', 'Laurent Ormond'),
            'lorem,laurent ormond,0000001405$068d$lo',
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_unknown_f.

This module contains unit tests for abydos.distance.UnknownF
"""

import unittest

from abydos.distance import UnknownF


class UnknownFTestCases(unittest.TestCase):
    """Test UnknownF functions.

    abydos.distance.UnknownF
    """

    cmp = UnknownF()
    cmp_no_d = UnknownF(alphabet=0)

    def test_unknown_f_sim_score(self):
        """Test abydos.distance.UnknownF.sim_score."""
        # Base cases
        self.assertEqual(self.cmp.sim_score('', ''), 1.0)
        self.assertEqual(self.cmp.sim_score('a', ''), 0.0)
        self.assertEqual(self.cmp.sim_score('', 'a'), 0.0)
        self.assertEqual(self.cmp.sim_score('abc', ''), 0.0)
        self.assertEqual(self.cmp.sim_score('', 'abc'), 0.0)
        self.assertEqual(self.cmp.sim_score('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp.sim_score('abcd', 'efgh'), 1.0)

        self.assertAlmostEqual(
            self.cmp.sim_score('Nigel', 'Niall'), 0.3068528194
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('Niall', 'Nigel'), 0.3068528194
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('Colin', 'Coiln'), 0.3068528194
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('Coiln', 'Colin'), 0.3068528194
        )
        self.assertAlmostEqual(
            self.cmp.sim_score('ATCAACGAGT', 'AACGATTAG'), 0.5956699662
        )

        # Exceptions
        self.assertRaises(NotImplementedError, self.cmp.sim, 'a', 'a')
        self.assertRaises(NotImplementedError, self.cmp.dist, 'a', 'a')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_reth_schek.

This module contains unit tests for abydos.phonetic.RethSchek
"""

import unittest

from abydos.phonetic import RethSchek


class RethSchekTestCases(unittest.TestCase):
    """Test Reth-Schek Phonetik functions.

    test cases for abydos.phonetic.RethSchek
    """

    pa = RethSchek()

    def test_reth_schek_phonetik(self):
        """Test abydos.phonetic.RethSchek."""
        # Base cases
        self.assertEqual(self.pa.encode(''), '')

        # equivalents
        self.assertEqual(self.pa.encode('Häschen'), self.pa.encode('Haeschen'))
        self.assertEqual(self.pa.encode('Schloß'), self.pa.encode('Schloss'))
        self.assertEqual(self.pa.encode('üben'), self.pa.encode('ueben'))
        self.assertEqual(
            self.pa.encode('Eichörnchen'), self.pa.encode('Eichoernchen')
        )

        self.assertEqual(self.pa.encode('Häschen'), 'HESCHEN')
        self.assertEqual(self.pa.encode('Eichörnchen'), 'AIGHOERNGHEN')
        self.assertEqual(self.pa.encode('Hexe'), 'HEXE')
        self.assertEqual(self.pa.encode('Chemie'), 'GHEMI')
        self.assertEqual(self.pa.encode('Brille'), 'BRILE')
        self.assertEqual(self.pa.encode('Brilleille'), 'BRILAILE')
        self.assertEqual(self.pa.encode('Niveau'), 'NIFEAU')
        self.assertEqual(self.pa.encode('Korb'), 'GORB')
        self.assertEqual(self.pa.encode('Heino'), 'HAINO')
        self.assertEqual(self.pa.encode('Nekka'), 'NEKA')
        self.assertEqual(self.pa.encode('Aleph'), 'ALEF')
        self.assertEqual(self.pa.encode('Aleppo'), 'ALEBO')
        self.assertEqual(self.pa.encode('Endzipfel'), 'ENDZIBFL')
        self.assertEqual(self.pa.encode('verbrandt'), 'FERBRAND')
        self.assertEqual(self.pa.encode('Cent'), 'GEND')
        self.assertEqual(self.pa.encode('addiscendae'), 'ADISGENDE')
        self.assertEqual(self.pa.encode('kickx'), 'GIGX')
        self.assertEqual(self.pa.encode('sanctionen'), 'SANGDIONEN')
        self.assertEqual(self.pa.encode('Kuh'), 'GU')
        self.assertEqual(self.pa.encode('lecker'), 'LEGR')
        self.assertEqual(self.pa.encode('rödlich'), 'ROEDLIG')


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.tokenizer.test_tokenizer_cv_cluster.

This module contains unit tests for abydos.tokenizer.CVClusterTokenizer
"""

import unittest

from abydos.tokenizer import CVClusterTokenizer


class CVClusterTokenizerTestCases(unittest.TestCase):
    """Test abydos.tokenizer.CVClusterTokenizer."""

    def test_cv_cluster_tokenizer(self):
        """Test abydos.tokenizer.CVClusterTokenizer."""
        self.assertEqual(
            sorted(CVClusterTokenizer().tokenize('').get_list()), []
        )
        self.assertEqual(
            sorted(CVClusterTokenizer().tokenize('a').get_list()), ['a']
        )

        tok = CVClusterTokenizer()

        self.assertEqual(
            sorted(tok.tokenize('nelson').get_list()),
            sorted(['ne', 'lso', 'n']),
        )
        self.assertEqual(
            sorted(tok.tokenize('neilson').get_list()),
            sorted(['nei', 'lso', 'n']),
        )
        self.assertEqual(
            sorted(tok.tokenize('peninsular').get_list()),
            sorted(['pe', 'ni', 'nsu', 'la', 'r']),
        )
        self.assertEqual(
            sorted(tok.tokenize('spectacular').get_list()),
            sorted(['spe', 'cta', 'cu', 'la', 'r']),
        )
        self.assertEqual(
            sorted(tok.tokenize('sufficiently').get_list()),
            sorted(['su', 'ffi', 'cie', 'ntly']),
        )
        self.assertEqual(
            sorted(tok.tokenize('yachting').get_list()),
            sorted(['ya', 'chti', 'ng']),
        )
        self.assertEqual(
            sorted(tok.tokenize('caterpillars').get_list()),
            sorted(['ca', 'te', 'rpi', 'lla', 'rs']),
        )
        self.assertEqual(
            sorted(tok.tokenize('Götterdämmerung').get_list()),
            sorted(['Gö', 'tte', 'rdä', 'mme', 'ru', 'ng']),
        )

        tok = CVClusterTokenizer(consonants='ptkbdgmn', vowels='aeiouwy')
        self.assertEqual(
            sorted(tok.tokenize('#winning #losing').get_list()),
            sorted(['#', 'wi', 'nni', 'ng', '#', 'losing']),
        )


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""call_and_write_log.py.

This helper script takes one argument, a call to pylint, pycodestyle,
flake8, or doc8. It captures stdout and writes it to a log file.

The reason for this script to exist is as a workaround for tox
not supporting writing to files, even though I want it to do that
to maintain logs & create badges.

Although this isn't a terribly secure script, it's only used in development and
not included as part of the distributed Abydos package itself. Accordingly,
I'm suppressing security warnings from flake8.
"""

import sys
from subprocess import call  # noqa: S404


def _run_script():
    const_ret = False
    if len(sys.argv) > 2:
        try:
            const_ret = int(sys.argv[2])
        except ValueError:
            pass

    if len(sys.argv) > 1:
        args = sys.argv[1].split()
        if args[0] not in {
            'pylint',
            'pycodestyle',
            'flake8',
            'doc8',
            'pydocstyle',
            'sloccount',
        }:
            sys.exit(1)
        with open(args[0] + '.log', 'w') as output:
            retval = call(args, stdout=output, shell=False)  # noqa: S603
            if args[0] in {'pylint', 'pycodestyle'}:
                retval = 0
        sys.exit(const_ret if const_ret else retval)


if __name__ == '__main__':
    _run_script()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stats.test_stats_mean.

This module contains unit tests for abydos.stats mean functions
"""

import unittest
from math import isnan

from abydos.stats import (
    aghmean,
    agmean,
    amean,
    cmean,
    ghmean,
    gmean,
    heronian_mean,
    hmean,
    hoelder_mean,
    imean,
    lehmer_mean,
    lmean,
    median,
    midrange,
    mode,
    qmean,
    seiffert_mean,
    std,
    var,
)


class MeansTestCases(unittest.TestCase):
    """Test abydos.stats mean functions."""

    _ones = [1, 1, 1, 1, 1]
    _zeros = [0, 0, 0, 0, 0]
    _one_to_five = [1, 2, 3, 4, 5]
    _onethreefive = [1, 1, 3, 5, 5]
    _floats = [0.5, 0.8, 0.1, 0.2, 0.25]
    _has_inf = [0, 1, 2, float('inf')]
    _2ones = [1, 1]
    _2zeros = [0, 0]
    _onetwo = [1, 2]
    _2floats = [0.5, 0.25]

    def test_means_amean(self):
        """Test abydos.stats.amean."""
        self.assertAlmostEqual(amean(self._ones), 1)
        self.assertAlmostEqual(amean(self._zeros), 0)
        self.assertAlmostEqual(amean(self._one_to_five), 3)
        self.assertAlmostEqual(amean(self._onethreefive), 3)
        self.assertAlmostEqual(amean(self._floats), 0.37)

    def test_means_gmean(self):
        """Test abydos.stats.gmean."""
        self.assertAlmostEqual(gmean(self._ones), 1)
        self.assertAlmostEqual(gmean(self._zeros), 0)
        self.assertAlmostEqual(gmean(self._one_to_five), 2.605171084697352)
        self.assertAlmostEqual(gmean(self._onethreefive), 2.3714406097793117)
        self.assertAlmostEqual(gmean(self._floats), 0.2885399811814427)

    def test_means_hmean(self):
        """Test abydos.stats.hmean."""
        self.assertAlmostEqual(hmean(self._ones), 1)
        self.assertAlmostEqual(hmean(self._zeros), 0)
        self.assertAlmostEqual(hmean(self._one_to_five), 2.18978102189781)
        self.assertAlmostEqual(hmean(self._onethreefive), 1.8292682926829265)
        self.assertAlmostEqual(hmean(self._floats), 0.2247191011235955)
        self.assertEqual(hmean([0, 1, 2]), 0)
        self.assertEqual(hmean([1, 2, 3, 0]), 0)
        self.assertTrue(isnan(hmean([0, 0, 1, 2])))
        self.assertTrue(isnan(hmean([1, 0, 2, 0, 3])))
        self.assertTrue(isnan(hmean([1, 0, 2, 0, 3, 0])))
        self.assertTrue(isnan(hmean([1, 0, 2, 0, 3, 0, 0])))
        self.assertEqual(hmean([0, 0]), 0)
        self.assertEqual(hmean([5, 5, 5, 5, 5]), 5)
        self.assertEqual(hmean([0]), 0)
        self.assertEqual(hmean([8]), 8)
        self.assertRaises(ValueError, hmean, ([]))

    def test_means_qmean(self):
        """Test abydos.stats.qmean."""
        self.assertAlmostEqual(qmean(self._ones), 1)
        self.assertAlmostEqual(qmean(self._zeros), 0)
        self.assertAlmostEqual(qmean(self._one_to_five), 3.3166247903554)
        self.assertAlmostEqual(qmean(self._onethreefive), 3.492849839314596)
        self.assertAlmostEqual(qmean(self._floats), 0.4477722635447623)

    def test_means_cmean(self):
        """Test abydos.stats.cmean."""
        self.assertAlmostEqual(cmean(self._ones), 1)
        self.assertAlmostEqual(cmean(self._one_to_five), 3.6666666666666665)
        self.assertAlmostEqual(cmean(self._onethreefive), 4.066666666666666)
        self.assertAlmostEqual(cmean(self._floats), 0.5418918918918919)

    def test_means_lmean(self):
        """Test abydos.stats.lmean."""
        self.assertAlmostEqual(lmean(self._one_to_five), 2.6739681320855766)
        self.assertAlmostEqual(lmean(self._floats), 0.301387278840469)
        self.assertEqual(lmean([1, 1]), 1.0)
        self.assertEqual(lmean([2, 2]), 2.0)
        self.assertEqual(lmean([2, 0]), 0.0)
        self.assertAlmostEqual(lmean([1, 2]), 1.4426950408889634)
        self.assertRaises(ValueError, lmean, (1, 1, 1))
        self.assertRaises(ValueError, lmean, (0.15, 0.15, 1))

    def test_means_imean(self):
        """Test abydos.stats.imean."""
        self.assertRaises(ValueError, imean, self._ones)
        self.assertRaises(ValueError, imean, self._zeros)
        self.assertRaises(ValueError, imean, self._one_to_five)
        self.assertRaises(ValueError, imean, self._onethreefive)
        self.assertRaises(ValueError, imean, self._floats)
        self.assertAlmostEqual(imean(self._2ones), 1)
        self.assertTrue(isnan(imean(self._2zeros)))
        self.assertAlmostEqual(imean(self._onetwo), 1.4715177646857693)
        self.assertAlmostEqual(imean(self._2floats), 0.36787944117144233)
        self.assertEqual(imean([1]), 1)
        self.assertEqual(imean([0.05]), 0.05)

    def test_means_seiffert_mean(self):
        """Test abydos.stats.seiffert_mean."""
        self.assertRaises(ValueError, seiffert_mean, self._ones)
        self.assertRaises(ValueError, seiffert_mean, self._zeros)
        self.assertRaises(ValueError, seiffert_mean, self._one_to_five)
        self.assertRaises(ValueError, seiffert_mean, self._onethreefive)
        self.assertRaises(ValueError, seiffert_mean, self._floats)
        self.assertAlmostEqual(seiffert_mean(self._onetwo), 1.4712939827611637)
        self.assertAlmostEqual(
            seiffert_mean(self._2floats), 0.36782349569029094
        )
        self.assertEqual(seiffert_mean([1]), 1)
        self.assertEqual(seiffert_mean([0.05]), 0.05)
        self.assertTrue(isnan(seiffert_mean([1, 1])))

    def test_means_lehmer_mean(self):
        """Test abydos.stats.lehmer_mean."""
        self.assertAlmostEqual(lehmer_mean(self._ones), 1)
        self.assertAlmostEqual(
            lehmer_mean(self._one_to_five), 3.6666666666666665
        )
        self.assertAlmostEqual(
            lehmer_mean(self._onethreefive), 4.066666666666666
        )
        self.assertAlmostEqual(lehmer_mean(self._floats), 0.5418918918918919)

    def test_means_heronian_mean(self):
        """Test abydos.stats.heronian_mean."""
        self.assertAlmostEqual(heronian_mean(self._ones), 1)
        self.assertAlmostEqual(heronian_mean(self._zeros), 0)
        self.assertAlmostEqual(
            heronian_mean(self._one_to_five), 2.8421165194322837
        )
        self.assertAlmostEqual(
            heronian_mean(self._onethreefive), 2.7436226811701165
        )
        self.assertAlmostEqual(
            heronian_mean(self._floats), 0.33526945542427006
        )

    def test_means_hoelder_mean(self):
        """Test abydos.stats.hoelder_mean."""
        self.assertAlmostEqual(hoelder_mean(self._ones), 1)
        self.assertAlmostEqual(hoelder_mean(self._zeros), 0)
        self.assertAlmostEqual(
            hoelder_mean(self._one_to_five), 3.3166247903554
        )
        self.assertAlmostEqual(
            hoelder_mean(self._onethreefive), 3.492849839314596
        )
        self.assertAlmostEqual(hoelder_mean(self._floats), 0.4477722635447623)
        self.assertAlmostEqual(
            hoelder_mean(self._floats, 0), gmean(self._floats)
        )

    def test_means_agmean(self):
        """Test abydos.stats.agmean."""
        self.assertAlmostEqual(agmean(self._ones), 1)
        self.assertAlmostEqual(agmean(self._zeros), 0)
        self.assertAlmostEqual(agmean(self._one_to_five), 2.799103662640505)
        self.assertAlmostEqual(agmean(self._onethreefive), 2.6764865062631356)
        self.assertAlmostEqual(agmean(self._floats), 0.32800436242611486)
        self.assertTrue(isnan(agmean(self._has_inf)))

    def test_means_ghmean(self):
        """Test abydos.stats.ghmean."""
        self.assertAlmostEqual(ghmean(self._ones), 1)
        self.assertAlmostEqual(ghmean(self._one_to_five), 2.3839666656453167)
        self.assertAlmostEqual(ghmean(self._onethreefive), 2.0740491019412035)
        self.assertAlmostEqual(ghmean(self._floats), 0.2536468771476393)
        self.assertTrue(isnan(ghmean(self._has_inf)))

    def test_means_aghmean(self):
        """Test abydos.stats.aghmean."""
        self.assertAlmostEqual(aghmean(self._ones), 1)
        self.assertAlmostEqual(aghmean(self._one_to_five), 2.5769530579812563)
        self.assertAlmostEqual(aghmean(self._onethreefive), 2.3520502484275387)
        self.assertAlmostEqual(aghmean(self._floats), 0.28841285333045547)
        self.assertTrue(isnan(aghmean(self._has_inf)))

    def test_means_midrange(self):
        """Test abydos.stats.midrange."""
        self.assertAlmostEqual(midrange(self._ones), 1)
        self.assertAlmostEqual(midrange(self._zeros), 0)
        self.assertAlmostEqual(midrange(self._one_to_five), 3)
        self.assertAlmostEqual(midrange(self._onethreefive), 3)
        self.assertAlmostEqual(midrange(self._floats), 0.45)

    def test_means_median(self):
        """Test abydos.stats.median."""
        self.assertAlmostEqual(median(self._ones), 1)
        self.assertAlmostEqual(median(self._zeros), 0)
        self.assertAlmostEqual(median(self._one_to_five), 3)
        self.assertAlmostEqual(median(self._onethreefive), 3)
        self.assertAlmostEqual(median(self._floats), 0.25)
        self.assertAlmostEqual(median([0, 2, 4, 8]), 3)
        self.assertAlmostEqual(median([0.01, 0.2, 0.4, 5]), 0.3)

    def test_means_mode(self):
        """Test abydos.stats.mode."""
        self.assertEqual(mode(self._ones), 1)
        self.assertEqual(mode(self._zeros), 0)
        self.assertEqual(mode([1, 1, 2, 2, 2]), 2)
        self.assertEqual(mode([1, 5, 5, 2, 5, 2]), 5)

    def test_means_var(self):
        """Test abydos.stats.var."""
        self.assertAlmostEqual(var(self._ones), 0)
        self.assertAlmostEqual(var(self._zeros), 0)
        self.assertAlmostEqual(var(self._one_to_five), 2)
        self.assertAlmostEqual(var(self._onethreefive), 3.2)

    def test_means_std(self):
        """Test abydos.stats.std."""
        self.assertAlmostEqual(std(self._ones), 0)
        self.assertAlmostEqual(std(self._zeros), 0)
        self.assertAlmostEqual(std(self._one_to_five), 2 ** 0.5)
        self.assertAlmostEqual(std(self._onethreefive), 3.2 ** 0.5)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.fingerprint.test_fingerprint_phonetic_fingerprint.

This module contains unit tests for abydos.fingerprint.Phonetic
"""

import unittest

from abydos.fingerprint import Phonetic
from abydos.phonetic import Phonet, Soundex


from .. import NIALL


class PhoneticTestCases(unittest.TestCase):
    """Test phonetic fingerprint functions.

    abydos.fingerprint.Phonetic
    """

    fp = Phonetic()
    fp_phonet = Phonetic(Phonet())
    fp_soundex = Phonetic(Soundex())
    soundex = Soundex()

    def test_phonetic_fingerprint(self):
        """Test abydos.fingerprint.Phonetic."""
        # Base case
        self.assertEqual(self.fp.fingerprint(''), '')

        self.assertEqual(
            self.fp.fingerprint(' '.join(NIALL)), 'a anl mknl njl nklk nl'
        )
        self.assertEqual(
            self.fp_phonet.fingerprint(' '.join(NIALL)),
            'knile makneil maknele neil nel nele nial nigeli '
            + 'nigl nil noigialach oneil ui',
        )
        self.assertEqual(
            self.fp_soundex.fingerprint(' '.join(NIALL)),
            'k540 m254 n240 n242 n400 o540 u000',
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stemmer.test_stemmer_lovins.

This module contains unit tests for abydos.stemmer.Lovins
"""

import codecs
import unittest

from abydos.stemmer import Lovins

from .. import _corpus_file


class LovinsTestCases(unittest.TestCase):
    """Test Lovins functions.

    abydos.stemmer.Lovins
    """

    stmr = Lovins()

    def test_lovins(self):
        """Test abydos.stemmer.Lovins."""
        # base case
        self.assertEqual(self.stmr.stem(''), '')

        # test cases from Lovins' "Development of a Stemming Algorithm":
        # http://www.mt-archive.info/MT-1968-Lovins.pdf
        self.assertEqual(self.stmr.stem('magnesia'), 'magnes')
        self.assertEqual(self.stmr.stem('magnesite'), 'magnes')
        self.assertEqual(self.stmr.stem('magnesian'), 'magnes')
        self.assertEqual(self.stmr.stem('magnesium'), 'magnes')
        self.assertEqual(self.stmr.stem('magnet'), 'magnet')
        self.assertEqual(self.stmr.stem('magnetic'), 'magnet')
        self.assertEqual(self.stmr.stem('magneto'), 'magnet')
        self.assertEqual(self.stmr.stem('magnetically'), 'magnet')
        self.assertEqual(self.stmr.stem('magnetism'), 'magnet')
        self.assertEqual(self.stmr.stem('magnetite'), 'magnet')
        self.assertEqual(self.stmr.stem('magnetitic'), 'magnet')
        self.assertEqual(self.stmr.stem('magnetizable'), 'magnet')
        self.assertEqual(self.stmr.stem('magnetization'), 'magnet')
        self.assertEqual(self.stmr.stem('magnetize'), 'magnet')
        self.assertEqual(self.stmr.stem('magnetometer'), 'magnetometer')
        self.assertEqual(self.stmr.stem('magnetometric'), 'magnetometer')
        self.assertEqual(self.stmr.stem('magnetometry'), 'magnetometer')
        self.assertEqual(self.stmr.stem('magnetomotive'), 'magnetomot')
        self.assertEqual(self.stmr.stem('magnetron'), 'magnetron')
        self.assertEqual(self.stmr.stem('metal'), 'metal')
        self.assertEqual(self.stmr.stem('metall'), 'metal')
        self.assertEqual(self.stmr.stem('metallically'), 'metal')
        self.assertEqual(self.stmr.stem('metalliferous'), 'metallifer')
        self.assertEqual(self.stmr.stem('metallize'), 'metal')
        self.assertEqual(self.stmr.stem('metallurgical'), 'metallurg')
        self.assertEqual(self.stmr.stem('metallurgy'), 'metallurg')
        self.assertEqual(self.stmr.stem('induction'), 'induc')
        self.assertEqual(self.stmr.stem('inductance'), 'induc')
        self.assertEqual(self.stmr.stem('induced'), 'induc')
        self.assertEqual(self.stmr.stem('angular'), 'angl')
        self.assertEqual(self.stmr.stem('angle'), 'angl')

        # missed branch test cases
        self.assertEqual(self.stmr.stem('feminism'), 'fem')

    def test_lovins_snowball(self):
        """Test abydos.stemmer.Lovins (Snowball testset).

        These test cases are from
        https://github.com/snowballstem/snowball-data/tree/master/lovins
        """
        #  Snowball Lovins test set
        with codecs.open(
            _corpus_file('snowball_lovins.csv'), encoding='utf-8'
        ) as snowball_ts:
            next(snowball_ts)
            for line in snowball_ts:
                if line[0] != '#':
                    line = line.strip().split(',')
                    word, stem = line[0], line[1]
                    self.assertEqual(self.stmr.stem(word), stem.lower())


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stemmer.test_stemmer_stemmer.

This module contains unit tests for abydos.stemmer._Stemmer
"""

import unittest

# noinspection PyProtectedMember
from abydos.stemmer._stemmer import _Stemmer


class SnowballTestCases(unittest.TestCase):
    """Test _Stemmer base class.

    abydos.stemmer._Stemmer
    """

    stmr = _Stemmer()

    def test__stemmer(self):
        """Test abydos.stemmer._Stemmer."""
        # base case
        self.assertEqual(self.stmr.stem(''), '')
        self.assertEqual(self.stmr.stem('word'), 'word')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic__phonetic.

This module contains unit tests for abydos.phonetic._Phonetic
"""

import unittest

from abydos.phonetic import Davidson

# noinspection PyProtectedMember
from abydos.phonetic._phonetic import _Phonetic


class PhoneticTestCases(unittest.TestCase):
    """Test _Phonetic base class.

    test cases for abydos.phonetic._Phonetic
    """

    pa = _Phonetic()
    dav = Davidson()

    def test_phonetic_delete_consecutive_repeats(self):
        """Test abydos.phonetic._Phonetic_delete_consecutive_repeats."""
        self.assertEqual(
            self.pa._delete_consecutive_repeats('REDDEE'), 'REDE'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._delete_consecutive_repeats('AEIOU'), 'AEIOU'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._delete_consecutive_repeats('AAACCCTTTGGG'),  # noqa: SF01
            'ACTG',
        )

    def test_phonetic_encode(self):
        """Test abydos.phonetic._Phonetic.encode."""
        self.assertEqual(self.pa.encode(''), '')
        self.assertEqual(self.pa.encode('word'), 'word')

    def test_phonetic_encode_alpha(self):
        """Test abydos.phonetic._Phonetic.encode_alpha."""
        self.assertEqual(self.pa.encode_alpha(''), '')
        self.assertEqual(self.pa.encode_alpha('word'), 'word')
        self.assertEqual(
            self.dav.encode_alpha('word'), self.dav.encode('word')
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.fingerprint.test_fingerprint_consonant.

This module contains unit tests for abydos.fingerprint.Consonant
"""

import unittest

from abydos.fingerprint import Consonant


class ConsonantTestCases(unittest.TestCase):
    """Test Taft's consonant coding functions.

    abydos.fingerprint.Consonant
    """

    def test_consonant_fingerprint(self):
        """Test abydos.fingerprint.Consonant."""
        # Base case
        self.assertEqual(Consonant().fingerprint(''), '')

        # Test cases from paper
        self.assertEqual(Consonant(variant=1).fingerprint('ABRAMS'), 'ABRMS')
        self.assertEqual(Consonant(variant=2).fingerprint('ARROW'), 'ARR')
        self.assertEqual(Consonant(variant=3).fingerprint('ABRAHAM'), 'ABRM')
        self.assertEqual(
            Consonant(variant=1, doubles=False).fingerprint('ARROW'), 'ARW'
        )
        self.assertEqual(
            Consonant(variant=2, doubles=False).fingerprint('ARROW'), 'AR'
        )
        self.assertEqual(
            Consonant(variant=3, doubles=False).fingerprint('GARRETH'), 'GRT'
        )

        # coverage
        self.assertEqual(Consonant(vowels='R').fingerprint('ARROW'), 'AOW')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.fingerprint.test_fingerprint_omission_key.

This module contains unit tests for abydos.fingerprint.OmissionKey
"""

import unittest

from abydos.fingerprint import OmissionKey


class OmissionKeyTestCases(unittest.TestCase):
    """Test OmissionKey functions.

    abydos.fingerprint.OmissionKey
    """

    fp = OmissionKey()

    def test_omission_key(self):
        """Test abydos.fingerprint.OmissionKey."""
        # Base case
        self.assertEqual(self.fp.fingerprint(''), '')

        # http://dl.acm.org/citation.cfm?id=358048
        self.assertEqual(self.fp.fingerprint('microelectronics'), 'MCLNTSRIOE')
        self.assertEqual(self.fp.fingerprint('circumstantial'), 'MCLNTSRIUA')
        self.assertEqual(self.fp.fingerprint('luminescent'), 'MCLNTSUIE')
        self.assertEqual(self.fp.fingerprint('multinucleate'), 'MCLNTUIEA')
        self.assertEqual(self.fp.fingerprint('multinucleon'), 'MCLNTUIEO')
        self.assertEqual(self.fp.fingerprint('cumulene'), 'MCLNUE')
        self.assertEqual(self.fp.fingerprint('luminance'), 'MCLNUIAE')
        self.assertEqual(self.fp.fingerprint('coelomic'), 'MCLOEI')
        self.assertEqual(self.fp.fingerprint('molecule'), 'MCLOEU')
        self.assertEqual(self.fp.fingerprint('cameral'), 'MCLRAE')
        self.assertEqual(self.fp.fingerprint('caramel'), 'MCLRAE')
        self.assertEqual(self.fp.fingerprint('maceral'), 'MCLRAE')
        self.assertEqual(self.fp.fingerprint('lacrimal'), 'MCLRAI')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_alpha_sis.

This module contains unit tests for abydos.phonetic.AlphaSIS
"""

import unittest

from abydos.phonetic import AlphaSIS


class AlphaSISTestCases(unittest.TestCase):
    """Test Alpha-SIS functions.

    test cases for abydos.phonetic.AlphaSIS
    """

    pa = AlphaSIS()

    def test_alpha_sis_encode(self):
        """Test abydos.phonetic.AlphaSIS."""
        self.assertEqual(self.pa.encode(''), '00000000000000')

        self.assertEqual(self.pa.encode('Rodgers'), '04740000000000')
        self.assertEqual(self.pa.encode('Rogers'), '04740000000000')
        self.assertEqual(
            self.pa.encode('Kant'), '07210000000000,06210000000000'
        )
        self.assertEqual(self.pa.encode('Knuth'), '02100000000000')
        self.assertEqual(self.pa.encode('Harper'), '24940000000000')
        self.assertEqual(
            self.pa.encode('Collier'), '07540000000000,06540000000000'
        )
        self.assertEqual(
            self.pa.encode('Schultz'), '06500000000000,06510000000000'
        )
        self.assertEqual(self.pa.encode('Livingston'), '05827012000000')

        # tests of repeated letters
        self.assertEqual(
            self.pa.encode('Colllier'), '07554000000000,06554000000000'
        )
        self.assertEqual(
            self.pa.encode('Collllier'), '07554000000000,06554000000000'
        )
        self.assertEqual(
            self.pa.encode('Colllllier'), '07555400000000,06555400000000'
        )
        self.assertEqual(
            self.pa.encode('Collllllier'), '07555400000000,06555400000000'
        )
        self.assertEqual(
            self.pa.encode('Colalalier'), '07555400000000,06555400000000'
        )

        # max_length bounds tests
        self.assertEqual(
            AlphaSIS(max_length=-1).encode('Niall'),
            '0250000000000000000000000000000000000000000000000000000000000000',
        )
        self.assertEqual(AlphaSIS(max_length=0).encode('Niall'), '0250')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('Rogers'), 'RKR')
        self.assertEqual(self.pa.encode_alpha('Kant'), 'KNT,JNT')
        self.assertEqual(self.pa.encode_alpha('Knuth'), 'NT')
        self.assertEqual(self.pa.encode_alpha('Harper'), 'HRPR')


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.tokenizer.test_tokenizer_vc_cluster.

This module contains unit tests for abydos.tokenizer.VCClusterTokenizer
"""

import unittest

from abydos.tokenizer import VCClusterTokenizer


class VCClusterTokenizerTestCases(unittest.TestCase):
    """Test abydos.tokenizer.VCClusterTokenizer."""

    def test_vc_cluster_tokenizer(self):
        """Test abydos.tokenizer.VCClusterTokenizer."""
        self.assertEqual(
            sorted(VCClusterTokenizer().tokenize('').get_list()), []
        )
        self.assertEqual(
            sorted(VCClusterTokenizer().tokenize('a').get_list()), ['a']
        )

        tok = VCClusterTokenizer()

        self.assertEqual(
            sorted(tok.tokenize('nelson').get_list()),
            sorted(['n', 'els', 'on']),
        )
        self.assertEqual(
            sorted(tok.tokenize('neilson').get_list()),
            sorted(['n', 'eils', 'on']),
        )
        self.assertEqual(
            sorted(tok.tokenize('peninsular').get_list()),
            sorted(['p', 'en', 'ins', 'ul', 'ar']),
        )
        self.assertEqual(
            sorted(tok.tokenize('spectacular').get_list()),
            sorted(['sp', 'ect', 'ac', 'ul', 'ar']),
        )
        self.assertEqual(
            sorted(tok.tokenize('sufficiently').get_list()),
            sorted(['s', 'uff', 'ic', 'ientl', 'y']),
        )
        self.assertEqual(
            sorted(tok.tokenize('yachting').get_list()),
            sorted(['yacht', 'ing']),
        )
        self.assertEqual(
            sorted(tok.tokenize('caterpillars').get_list()),
            sorted(['c', 'at', 'erp', 'ill', 'ars']),
        )
        self.assertEqual(
            sorted(tok.tokenize('Götterdämmerung').get_list()),
            sorted(['G', 'ött', 'erd', 'ämm', 'er', 'ung']),
        )

        tok = VCClusterTokenizer(consonants='ptkbdgmn', vowels='aeiouwy')
        self.assertEqual(
            sorted(tok.tokenize('#winning #losing').get_list()),
            sorted(['#', 'winn', 'ing', '#', 'losing']),
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stemmer.test_stemmer_snowball.

This module contains unit tests for abydos.stemmer._Snowball
"""

import unittest

# noinspection PyProtectedMember
from abydos.stemmer._snowball import _Snowball


class SnowballTestCases(unittest.TestCase):
    """Test _Snowball functions.

    abydos.stemmer._Snowball
    """

    stmr = _Snowball()
    stmr._vowels = set('aeiouy')  # noqa: SF01
    stmr._codanonvowels = set("bcdfghjklmnpqrstvz'")  # noqa: SF01

    def test_has_vowel(self):
        """Test abydos.stemmer._Snowball._has_vowel."""
        # base case
        self.assertFalse(self.stmr._sb_has_vowel(''))  # noqa: SF01

        # False cases
        self.assertFalse(self.stmr._sb_has_vowel('b'))  # noqa: SF01
        self.assertFalse(self.stmr._sb_has_vowel('c'))  # noqa: SF01
        self.assertFalse(self.stmr._sb_has_vowel('bc'))  # noqa: SF01
        self.assertFalse(
            self.stmr._sb_has_vowel('bcdfghjklmnpqrstvwxYz')  # noqa: SF01
        )
        self.assertFalse(self.stmr._sb_has_vowel('Y'))  # noqa: SF01

        # True cases
        self.assertTrue(self.stmr._sb_has_vowel('a'))  # noqa: SF01
        self.assertTrue(self.stmr._sb_has_vowel('e'))  # noqa: SF01
        self.assertTrue(self.stmr._sb_has_vowel('ae'))  # noqa: SF01
        self.assertTrue(self.stmr._sb_has_vowel('aeiouy'))  # noqa: SF01
        self.assertTrue(self.stmr._sb_has_vowel('y'))  # noqa: SF01

        self.assertTrue(self.stmr._sb_has_vowel('ade'))  # noqa: SF01
        self.assertTrue(self.stmr._sb_has_vowel('cad'))  # noqa: SF01
        self.assertTrue(self.stmr._sb_has_vowel('add'))  # noqa: SF01
        self.assertTrue(self.stmr._sb_has_vowel('phi'))  # noqa: SF01
        self.assertTrue(self.stmr._sb_has_vowel('pfy'))  # noqa: SF01

        self.assertFalse(self.stmr._sb_has_vowel('pfY'))  # noqa: SF01

    def test_sb_r1(self):
        """Test abydos.stemmer._Snowball._sb_r1."""
        # base case
        self.assertEqual(self.stmr._sb_r1(''), 0)  # noqa: SF01

        # examples from http://snowball.tartarus.org/texts/r1r2.html
        self.assertEqual(self.stmr._sb_r1('beautiful'), 5)  # noqa: SF01
        self.assertEqual(self.stmr._sb_r1('beauty'), 5)  # noqa: SF01
        self.assertEqual(self.stmr._sb_r1('beau'), 4)  # noqa: SF01
        self.assertEqual(self.stmr._sb_r1('animadversion'), 2)  # noqa: SF01
        self.assertEqual(self.stmr._sb_r1('sprinkled'), 5)  # noqa: SF01
        self.assertEqual(self.stmr._sb_r1('eucharist'), 3)  # noqa: SF01

        self.assertEqual(
            self.stmr._sb_r1('eucharist', r1_prefixes={'ist'}), 3  # noqa: SF01
        )
        self.assertEqual(
            self.stmr._sb_r1('ist', r1_prefixes={'ist'}), 3  # noqa: SF01
        )

    def test_sb_r2(self):
        """Test abydos.stemmer._Snowball._sb_r2."""
        # base case
        self.assertEqual(self.stmr._sb_r2(''), 0)  # noqa: SF01

        # examples from http://snowball.tartarus.org/texts/r1r2.html
        self.assertEqual(self.stmr._sb_r2('beautiful'), 7)  # noqa: SF01
        self.assertEqual(self.stmr._sb_r2('beauty'), 6)  # noqa: SF01
        self.assertEqual(self.stmr._sb_r2('beau'), 4)  # noqa: SF01
        self.assertEqual(self.stmr._sb_r2('animadversion'), 4)  # noqa: SF01
        self.assertEqual(self.stmr._sb_r2('sprinkled'), 9)  # noqa: SF01
        self.assertEqual(self.stmr._sb_r2('eucharist'), 6)  # noqa: SF01

    def test_sb_ends_in_short_syllable(self):
        """Test abydos.stemmer._Snowball._sb_ends_in_short_syllable."""
        # base case
        self.assertFalse(
            self.stmr._sb_ends_in_short_syllable('')  # noqa: SF01
        )

        # examples from
        # http://snowball.tartarus.org/algorithms/english/stemmer.html
        self.assertTrue(
            self.stmr._sb_ends_in_short_syllable('rap')  # noqa: SF01
        )
        self.assertTrue(
            self.stmr._sb_ends_in_short_syllable('trap')  # noqa: SF01
        )
        self.assertTrue(
            self.stmr._sb_ends_in_short_syllable('entrap')  # noqa: SF01
        )
        self.assertTrue(
            self.stmr._sb_ends_in_short_syllable('ow')  # noqa: SF01
        )
        self.assertTrue(
            self.stmr._sb_ends_in_short_syllable('on')  # noqa: SF01
        )
        self.assertTrue(
            self.stmr._sb_ends_in_short_syllable('at')  # noqa: SF01
        )
        self.assertFalse(
            self.stmr._sb_ends_in_short_syllable('uproot')  # noqa: SF01
        )
        self.assertFalse(
            self.stmr._sb_ends_in_short_syllable('uproot')  # noqa: SF01
        )
        self.assertFalse(
            self.stmr._sb_ends_in_short_syllable('bestow')  # noqa: SF01
        )
        self.assertFalse(
            self.stmr._sb_ends_in_short_syllable('disturb')  # noqa: SF01
        )

        # missed branch test cases
        self.assertFalse(
            self.stmr._sb_ends_in_short_syllable('d')  # noqa: SF01
        )
        self.assertFalse(
            self.stmr._sb_ends_in_short_syllable('a')  # noqa: SF01
        )
        self.assertFalse(
            self.stmr._sb_ends_in_short_syllable('da')  # noqa: SF01
        )

    def test_sb_short_word(self):
        """Test abydos.stemmer._Snowball._sb_short_word."""
        # base case
        self.assertFalse(self.stmr._sb_short_word(''))  # noqa: SF01

        # examples from
        # http://snowball.tartarus.org/algorithms/english/stemmer.html
        self.assertTrue(self.stmr._sb_short_word('bed'))  # noqa: SF01
        self.assertTrue(self.stmr._sb_short_word('shed'))  # noqa: SF01
        self.assertTrue(self.stmr._sb_short_word('shred'))  # noqa: SF01
        self.assertFalse(self.stmr._sb_short_word('bead'))  # noqa: SF01
        self.assertFalse(self.stmr._sb_short_word('embed'))  # noqa: SF01
        self.assertFalse(self.stmr._sb_short_word('beds'))  # noqa: SF01


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.compression.test_compression_arithmetic.

This module contains unit tests for abydos.compression.Arithmetic
"""

import unittest
from fractions import Fraction

from abydos.compression import Arithmetic

from .. import NIALL


class ArithmeticCoderTestCases(unittest.TestCase):
    """Test abydos.compression.Arithmetic.train & .Arithmetic.encode."""

    niall_probs = {
        'a': (Fraction(41, 57), Fraction(91, 114)),
        ' ': (Fraction(25, 114), Fraction(7, 19)),
        'c': (Fraction(97, 114), Fraction(50, 57)),
        'e': (Fraction(29, 57), Fraction(73, 114)),
        "'": (Fraction(56, 57), Fraction(113, 114)),
        'g': (Fraction(47, 57), Fraction(97, 114)),
        '\x00': (Fraction(113, 114), Fraction(1, 1)),
        'i': (Fraction(73, 114), Fraction(41, 57)),
        'M': (Fraction(17, 19), Fraction(52, 57)),
        'K': (Fraction(37, 38), Fraction(56, 57)),
        'j': (Fraction(50, 57), Fraction(17, 19)),
        '\xed': (Fraction(91, 114), Fraction(47, 57)),
        'l': (Fraction(0, 1), Fraction(25, 114)),
        'o': (Fraction(53, 57), Fraction(107, 114)),
        'N': (Fraction(7, 19), Fraction(29, 57)),
        '\xe9': (Fraction(52, 57), Fraction(35, 38)),
        '\xe1': (Fraction(35, 38), Fraction(53, 57)),
        'U': (Fraction(109, 114), Fraction(55, 57)),
        'O': (Fraction(55, 57), Fraction(37, 38)),
        'h': (Fraction(18, 19), Fraction(109, 114)),
        'n': (Fraction(107, 114), Fraction(18, 19)),
    }

    coder = Arithmetic()

    def test_arithmetic_train(self):
        """Test abydos.compression.Arithmetic.train."""
        self.coder.train('')
        self.assertEqual(self.coder.get_probs(), {'\x00': (0, 1)})
        self.coder.train(' '.join(NIALL))
        self.assertEqual(self.coder.get_probs(), self.niall_probs)
        self.coder.train(' '.join(sorted(NIALL)))
        self.assertEqual(self.coder.get_probs(), self.niall_probs)

        self.coder.train(' '.join(NIALL))
        niall_probs_new = self.coder.get_probs()
        self.coder.train(' '.join(sorted(NIALL)))
        self.assertEqual(niall_probs_new, self.coder.get_probs())

        self.coder.train('\x00'.join(NIALL))
        self.assertEqual(niall_probs_new, self.coder.get_probs())

    def test_arithmetic_encode(self):
        """Test abydos.compression.Arithmetic.encode."""
        self.coder.set_probs(self.niall_probs)
        self.assertEqual(self.coder.encode(''), (254, 8))
        self.assertEqual(self.coder.encode('a'), (3268, 12))
        self.assertEqual(self.coder.encode('Niall'), (3911665, 23))
        self.assertEqual(self.coder.encode('Ni\x00ll'), (1932751, 22))
        self.assertEqual(self.coder.encode('Niel'), (486801, 20))
        self.assertEqual(self.coder.encode('Mean'), (243067161, 28))
        self.assertEqual(
            self.coder.encode('Neil Noígíallach'), (2133315320471368785758, 72)
        )
        self.assertRaises(KeyError, self.coder.encode, 'NIALL')
        self.coder.set_probs({'\x00': (0, 1)})
        self.assertEqual(self.coder.encode(''), (1, 1))

    def test_arithmetic_decode(self):
        """Test abydos.compression.Arithmetic.decode."""
        self.coder.set_probs(self.niall_probs)
        self.assertEqual(self.coder.decode(254, 8), '')
        self.assertEqual(self.coder.decode(3268, 12), 'a')
        self.assertEqual(self.coder.decode(3911665, 23), 'Niall')
        self.assertEqual(self.coder.decode(1932751, 22), 'Ni ll')
        self.assertEqual(self.coder.decode(486801, 20), 'Niel')
        self.assertEqual(self.coder.decode(243067161, 28), 'Mean')
        self.assertEqual(
            self.coder.decode(2133315320471368785758, 72), 'Neil Noígíallach'
        )
        self.coder.set_probs({})
        self.assertEqual(self.coder.decode(0, 0), '')
        self.coder.set_probs({'\x00': (0, 1)})
        self.assertEqual(self.coder.decode(1, 1), '')


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_discounted_levenshtein.

This module contains unit tests for abydos.distance.DiscountedLevenshtein
"""

import unittest

from abydos.distance import DiscountedLevenshtein


class DiscountedLevenshteinTestCases(unittest.TestCase):
    """Test DiscountedLevenshtein functions.

    abydos.distance.DiscountedLevenshtein
    """

    cmp = DiscountedLevenshtein()
    cmp_coda = DiscountedLevenshtein(discount_from='coda')

    def test_discounted_levenshtein_aligmnent(self):
        """Test abydos.distance.DiscountedLevenshtein.alignment."""
        self.assertEqual(self.cmp.alignment('', ''), (0, '', ''))

        self.assertEqual(self.cmp.alignment('a', 'a'), (0.0, 'a', 'a'))
        self.assertEqual(self.cmp.alignment('ab', 'ab'), (0.0, 'ab', 'ab'))
        self.assertEqual(self.cmp.alignment('', 'a'), (1.0, '-', 'a'))
        self.assertEqual(
            self.cmp.alignment('', 'ab'), (1.845793595028118, '--', 'ab')
        )
        self.assertEqual(self.cmp.alignment('a', 'c'), (1.0, 'a', 'c'))

        self.assertEqual(self.cmp.alignment('abc', 'ac'), (1.0, 'abc', 'a-c'))
        self.assertEqual(
            self.cmp.alignment('abbc', 'ac'),
            (1.845793595028118, 'abbc', 'a--c'),
        )
        self.assertEqual(
            self.cmp.alignment('abbc', 'abc'),
            (0.8457935950281179, 'abbc', 'ab-c'),
        )

        self.assertEqual(
            DiscountedLevenshtein(mode='osa').alignment('Niall', 'Naill'),
            (0.8457935950281179, 'Niall', 'Naill'),
        )

        self.assertEqual(
            self.cmp.alignment('abcd', 'efgh'),
            (3.594032108779918, 'abcd', 'efgh'),
        )

        self.assertEqual(
            self.cmp.alignment('Nigel', 'Niall'),
            (1.5940321087799176, 'Nigel', 'Niall'),
        )
        self.assertEqual(
            self.cmp.alignment('Niall', 'Nigel'),
            (1.5940321087799176, 'Niall', 'Nigel'),
        )
        self.assertEqual(
            self.cmp.alignment('Colin', 'Coiln'),
            (1.5940321087799176, 'Coli-n', 'Co-iln'),
        )
        self.assertEqual(
            self.cmp.alignment('Coiln', 'Colin'),
            (1.5940321087799176, 'Coil-n', 'Co-lin'),
        )
        self.assertEqual(
            self.cmp.alignment('cccColin', 'Colin'),
            (2.594032108779918, 'cccColin', '---Colin'),
        )
        self.assertEqual(
            self.cmp.alignment('Colin', 'cccColin'),
            (2.594032108779918, '---Colin', 'cccColin'),
        )

    def test_discounted_levenshtein_dist_abs(self):
        """Test abydos.distance.DiscountedLevenshtein.dist_abs."""
        # Base cases
        self.assertEqual(self.cmp.dist_abs('', ''), 0.0)
        self.assertEqual(self.cmp.dist_abs('a', ''), 1.0)
        self.assertEqual(self.cmp.dist_abs('', 'a'), 1.0)
        self.assertEqual(self.cmp.dist_abs('abc', ''), 2.845793595028118)
        self.assertEqual(self.cmp.dist_abs('', 'abc'), 2.845793595028118)
        self.assertEqual(self.cmp.dist_abs('abc', 'abc'), 0.0)
        self.assertEqual(self.cmp.dist_abs('abcd', 'efgh'), 3.594032108779918)

        self.assertAlmostEqual(
            self.cmp.dist_abs('Nigel', 'Niall'), 1.5940321087799176
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs('Niall', 'Nigel'), 1.5940321087799176
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs('Colin', 'Coiln'), 1.5940321087799176
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs('Coiln', 'Colin'), 1.5940321087799176
        )
        self.assertAlmostEqual(
            self.cmp.dist_abs('ATCAACGAGT', 'AACGATTAG'), 3.480037325627888
        )

    def test_discounted_levenshtein_dist(self):
        """Test abydos.distance.DiscountedLevenshtein.dist."""
        # Base cases
        self.assertEqual(self.cmp.dist('', ''), 0.0)
        self.assertEqual(self.cmp.dist('a', ''), 1.0)
        self.assertEqual(self.cmp.dist('', 'a'), 1.0)
        self.assertEqual(self.cmp.dist('abc', ''), 1.0)
        self.assertEqual(self.cmp.dist('', 'abc'), 1.0)
        self.assertEqual(self.cmp.dist('abc', 'abc'), 0.0)
        self.assertEqual(self.cmp.dist('abcd', 'efgh'), 1.0)

        self.assertAlmostEqual(
            self.cmp.dist('Nigel', 'Niall'), 0.3729338516783247
        )
        self.assertAlmostEqual(
            self.cmp.dist('Niall', 'Nigel'), 0.3729338516783247
        )
        self.assertAlmostEqual(
            self.cmp.dist('Colin', 'Coiln'), 0.3729338516783247
        )
        self.assertAlmostEqual(
            self.cmp.dist('Coiln', 'Colin'), 0.3729338516783247
        )
        self.assertAlmostEqual(
            self.cmp.dist('ATCAACGAGT', 'AACGATTAG'), 0.49025364879218414
        )

        self.assertEqual(self.cmp_coda.dist('', ''), 0.0)
        self.assertEqual(self.cmp_coda.dist('a', ''), 1.0)
        self.assertEqual(self.cmp_coda.dist('', 'a'), 1.0)
        self.assertEqual(self.cmp_coda.dist('abc', ''), 1.0)
        self.assertEqual(self.cmp_coda.dist('', 'abc'), 1.0)
        self.assertEqual(self.cmp_coda.dist('abc', 'abc'), 0.0)
        self.assertEqual(self.cmp_coda.dist('abcd', 'efgh'), 1.0)

        self.assertAlmostEqual(
            self.cmp_coda.dist('Nigel', 'Niall'), 0.43183503707707127
        )
        self.assertAlmostEqual(
            self.cmp_coda.dist('Niall', 'Nigel'), 0.43183503707707127
        )
        self.assertAlmostEqual(
            self.cmp_coda.dist('Colin', 'Coiln'), 0.43183503707707127
        )
        self.assertAlmostEqual(
            self.cmp_coda.dist('Coiln', 'Colin'), 0.43183503707707127
        )
        self.assertAlmostEqual(
            self.cmp_coda.dist('ATCAACGAGT', 'AACGATTAG'), 0.49025364879218414
        )

        # coverage additions
        self.assertAlmostEqual(
            DiscountedLevenshtein(discount_func='exp').dist('Nigel', 'Niall'),
            0.3776202500185377,
        )
        self.assertAlmostEqual(
            DiscountedLevenshtein(
                discount_func=lambda x: 1 / (x + 2) ** 0.3
            ).dist('Nigel', 'Niall'),
            0.3808786265263343,
        )
        self.assertAlmostEqual(
            self.cmp_coda.dist('da', 'dde'), 0.7027916583599728
        )
        self.assertAlmostEqual(
            self.cmp_coda.dist('d', 'dd'), 0.42289679751405895
        )
        self.assertAlmostEqual(
            DiscountedLevenshtein(discount_from='invalid value').dist(
                'Nigel', 'Niall'
            ),
            0.3729338516783247,
        )
        self.assertAlmostEqual(
            DiscountedLevenshtein(mode='osa').dist('Nigel', 'Ngiall'),
            0.4534644632194963,
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_nrl.

This module contains unit tests for abydos.phonetic.NRL
"""

import unittest

from abydos.phonetic import NRL


class NRLTestCases(unittest.TestCase):
    """Test Naval Research Laboratory phonetic encoding functions.

    test cases for abydos.phonetic.NRL
    """

    pa = NRL()

    def test_nrl(self):
        """Test abydos.phonetic.NRL."""
        # Base case
        self.assertEqual(self.pa.encode(''), '')

        # common English words
        self.assertEqual(self.pa.encode('the'), 'DHAX')
        self.assertEqual(self.pa.encode('of'), 'AXv')
        self.assertEqual(self.pa.encode('and'), 'AEnd')
        self.assertEqual(self.pa.encode('a'), 'AX')
        self.assertEqual(self.pa.encode('to'), 'tUW')
        self.assertEqual(self.pa.encode('in'), 'IHn')
        self.assertEqual(self.pa.encode('is'), 'IHz')
        self.assertEqual(self.pa.encode('you'), 'yUW')
        self.assertEqual(self.pa.encode('that'), 'DHAEt')
        self.assertEqual(self.pa.encode('it'), 'IHt')
        self.assertEqual(self.pa.encode('he'), 'hIY')
        self.assertEqual(self.pa.encode('was'), 'wAAz')
        self.assertEqual(self.pa.encode('for'), 'fAOr')
        self.assertEqual(self.pa.encode('on'), 'AAn')
        self.assertEqual(self.pa.encode('are'), 'AAr')
        self.assertEqual(self.pa.encode('as'), 'AEz')
        self.assertEqual(self.pa.encode('with'), 'wIHTH')
        self.assertEqual(self.pa.encode('his'), 'hIHz')
        self.assertEqual(self.pa.encode('they'), 'DHEY')
        self.assertEqual(self.pa.encode('I'), 'AY')
        self.assertEqual(self.pa.encode('at'), 'AEt')
        self.assertEqual(self.pa.encode('be'), 'bIY')
        self.assertEqual(self.pa.encode('this'), 'DHIHs')
        self.assertEqual(self.pa.encode('have'), 'hAEv')
        self.assertEqual(self.pa.encode('from'), 'frAAm')
        self.assertEqual(self.pa.encode('or'), 'AOr')
        self.assertEqual(self.pa.encode('one'), 'wAHn')
        self.assertEqual(self.pa.encode('had'), 'hAEd')
        self.assertEqual(self.pa.encode('by'), 'bAY')
        self.assertEqual(self.pa.encode('word'), 'wERd')
        self.assertEqual(self.pa.encode('but'), 'bAHt')
        self.assertEqual(self.pa.encode('not'), 'nAAt')
        self.assertEqual(self.pa.encode('what'), 'WHAAt')
        self.assertEqual(self.pa.encode('all'), 'AOl')
        self.assertEqual(self.pa.encode('were'), 'wER')
        self.assertEqual(self.pa.encode('we'), 'wIY')
        self.assertEqual(self.pa.encode('when'), 'WHEHn')
        self.assertEqual(self.pa.encode('your'), 'yUWr')
        self.assertEqual(self.pa.encode('can'), 'kAEn')
        self.assertEqual(self.pa.encode('said'), 'sEHd')
        self.assertEqual(self.pa.encode('there'), 'DHEHr')
        self.assertEqual(self.pa.encode('use'), 'yUWz')
        self.assertEqual(self.pa.encode('an'), 'AEn')
        self.assertEqual(self.pa.encode('each'), 'IYCH')
        self.assertEqual(self.pa.encode('which'), 'WHIHCH')
        self.assertEqual(self.pa.encode('she'), 'SHIY')
        self.assertEqual(self.pa.encode('do'), 'dUW')
        self.assertEqual(self.pa.encode('how'), 'hAW')
        self.assertEqual(self.pa.encode('their'), 'DHEHr')
        self.assertEqual(self.pa.encode('if'), 'IHf')
        self.assertEqual(self.pa.encode('will'), 'wIHl')
        self.assertEqual(self.pa.encode('up'), 'AHp')
        self.assertEqual(self.pa.encode('other'), 'AHDHER')
        self.assertEqual(self.pa.encode('about'), 'AEbAWt')
        self.assertEqual(self.pa.encode('out'), 'AWt')
        self.assertEqual(self.pa.encode('many'), 'mEHnIY')
        self.assertEqual(self.pa.encode('then'), 'DHEHn')
        self.assertEqual(self.pa.encode('them'), 'DHEHm')
        self.assertEqual(self.pa.encode('these'), 'DHIYz')
        self.assertEqual(self.pa.encode('so'), 'sOW')
        self.assertEqual(self.pa.encode('some'), 'sAHm')
        self.assertEqual(self.pa.encode('her'), 'hER')
        self.assertEqual(self.pa.encode('would'), 'wUHd')
        self.assertEqual(self.pa.encode('make'), 'mEYk')
        self.assertEqual(self.pa.encode('like'), 'lAYk')
        self.assertEqual(self.pa.encode('him'), 'hIHm')
        self.assertEqual(self.pa.encode('into'), 'IHntUW')
        self.assertEqual(self.pa.encode('time'), 'tAYm')
        self.assertEqual(self.pa.encode('has'), 'hAEz')
        self.assertEqual(self.pa.encode('look'), 'lUHk')
        self.assertEqual(self.pa.encode('two'), 'tUW')
        self.assertEqual(self.pa.encode('more'), 'mAOr')
        self.assertEqual(self.pa.encode('write'), 'rAYt')
        self.assertEqual(self.pa.encode('go'), 'gOW')
        self.assertEqual(self.pa.encode('see'), 'sIY')
        self.assertEqual(self.pa.encode('number'), 'nAHmbER')
        self.assertEqual(self.pa.encode('no'), 'nOW')
        self.assertEqual(self.pa.encode('way'), 'wEY')
        self.assertEqual(self.pa.encode('could'), 'kUHd')
        self.assertEqual(self.pa.encode('people'), 'pIYpl')
        self.assertEqual(self.pa.encode('my'), 'mAY')
        self.assertEqual(self.pa.encode('than'), 'DHAEn')
        self.assertEqual(self.pa.encode('first'), 'fERst')
        self.assertEqual(self.pa.encode('water'), 'wAAtER')
        self.assertEqual(self.pa.encode('been'), 'bIYn')
        self.assertEqual(self.pa.encode('call'), 'kAOl')
        self.assertEqual(self.pa.encode('who'), 'hUW')
        self.assertEqual(self.pa.encode('oil'), 'OYl')
        self.assertEqual(self.pa.encode('its'), 'IHtz')
        self.assertEqual(self.pa.encode('now'), 'nAW')
        self.assertEqual(self.pa.encode('find'), 'fAYnd')
        self.assertEqual(self.pa.encode('long'), 'lAONG')
        self.assertEqual(self.pa.encode('down'), 'dAWn')
        self.assertEqual(self.pa.encode('day'), 'dEY')
        self.assertEqual(self.pa.encode('did'), 'dIHd')
        self.assertEqual(self.pa.encode('get'), 'gEHt')
        self.assertEqual(self.pa.encode('come'), 'kAHm')
        self.assertEqual(self.pa.encode('made'), 'mEYd')
        self.assertEqual(self.pa.encode('may'), 'mEY')
        self.assertEqual(self.pa.encode('part'), 'pAArt')
        self.assertEqual(self.pa.encode('supply'), 'sAHpplIH')
        self.assertEqual(self.pa.encode('corner'), 'kAOrnER')
        self.assertEqual(self.pa.encode('electric'), 'IYlEHktrIHk')
        self.assertEqual(self.pa.encode('insects'), 'IHnsEHktz')
        self.assertEqual(self.pa.encode('crops'), 'krAAps')
        self.assertEqual(self.pa.encode('tone'), 'tOWn')
        self.assertEqual(self.pa.encode('hit'), 'hIHt')
        self.assertEqual(self.pa.encode('sand'), 'sAEnd')
        self.assertEqual(self.pa.encode('doctor'), 'dAAktER')
        self.assertEqual(self.pa.encode('provide'), 'prAHvAYd')
        self.assertEqual(self.pa.encode('thus'), 'DHAHs')
        self.assertEqual(self.pa.encode("won't"), 'wOWnt')
        self.assertEqual(self.pa.encode('cook'), 'kUHk')
        self.assertEqual(self.pa.encode('bones'), 'bOWnz')
        self.assertEqual(self.pa.encode('tail'), 'tEYl')
        self.assertEqual(self.pa.encode('board'), 'bOWrd')
        self.assertEqual(self.pa.encode('modern'), 'mOWdERn')
        self.assertEqual(self.pa.encode('compound'), 'kAAmpAWnd')
        self.assertEqual(self.pa.encode('mine'), 'mAYn')
        self.assertEqual(self.pa.encode("wasn't"), 'wAAzAXnt')
        self.assertEqual(self.pa.encode('fit'), 'fIHt')
        self.assertEqual(self.pa.encode('addition'), 'AEddIHSHAXn')
        self.assertEqual(self.pa.encode('belong'), 'bIHlAONG')
        self.assertEqual(self.pa.encode('safe'), 'sEYf')
        self.assertEqual(self.pa.encode('soldiers'), 'sOWldIYERs')
        self.assertEqual(self.pa.encode('guess'), 'gEHs')
        self.assertEqual(self.pa.encode('silent'), 'sAYlEHnt')
        self.assertEqual(self.pa.encode('trade'), 'trEYd')
        self.assertEqual(self.pa.encode('rather'), 'rAEDHER')
        self.assertEqual(self.pa.encode('compare'), 'kAAmpEHr')
        self.assertEqual(self.pa.encode('crowd'), 'krOWd')
        self.assertEqual(self.pa.encode('poem'), 'pOWEHm')
        self.assertEqual(self.pa.encode('enjoy'), 'EHnjOY')
        self.assertEqual(self.pa.encode('elements'), 'IYlIYmEHntz')
        self.assertEqual(self.pa.encode('indicate'), 'IHndIHkEYt')
        self.assertEqual(self.pa.encode('except'), 'EHkssEHpt')
        self.assertEqual(self.pa.encode('expect'), 'EHkspEHkt')
        self.assertEqual(self.pa.encode('flat'), 'flAEt')
        self.assertEqual(self.pa.encode('seven'), 'sIYvEHn')
        self.assertEqual(self.pa.encode('interest'), 'IHntIYrEHst')
        self.assertEqual(self.pa.encode('sense'), 'sEHns')
        self.assertEqual(self.pa.encode('string'), 'strIHNG')
        self.assertEqual(self.pa.encode('blow'), 'blOW')
        self.assertEqual(self.pa.encode('famous'), 'fAEmAXs')
        self.assertEqual(self.pa.encode('value'), 'vAElUW')
        self.assertEqual(self.pa.encode('wings'), 'wIHNGz')
        self.assertEqual(self.pa.encode('movement'), 'mUWvIYmEHnt')
        self.assertEqual(self.pa.encode('pole'), 'pOWl')
        self.assertEqual(self.pa.encode('exciting'), 'EHkssAYtIHNG')
        self.assertEqual(self.pa.encode('branches'), 'brAEnCHIHz')
        self.assertEqual(self.pa.encode('thick'), 'THIHk')
        self.assertEqual(self.pa.encode('blood'), 'blUHd')
        self.assertEqual(self.pa.encode('lie'), 'lAY')
        self.assertEqual(self.pa.encode('spot'), 'spAAt')
        self.assertEqual(self.pa.encode('bell'), 'bEHl')
        self.assertEqual(self.pa.encode('fun'), 'fAHn')
        self.assertEqual(self.pa.encode('loud'), 'lAWd')
        self.assertEqual(self.pa.encode('consider'), 'kAAnsAYdER')
        self.assertEqual(self.pa.encode('suggested'), 'sAHgjEHstIHd')
        self.assertEqual(self.pa.encode('thin'), 'THIHn')
        self.assertEqual(self.pa.encode('position'), 'pAAzIHSHAXn')
        self.assertEqual(self.pa.encode('entered'), 'EHntIYrd')
        self.assertEqual(self.pa.encode('fruit'), 'frUWIHt')
        self.assertEqual(self.pa.encode('tied'), 'tAYd')
        self.assertEqual(self.pa.encode('rich'), 'rIHCH')
        self.assertEqual(self.pa.encode('dollars'), 'dAAlAArs')
        self.assertEqual(self.pa.encode('send'), 'sEHnd')
        self.assertEqual(self.pa.encode('sight'), 'sAYt')
        self.assertEqual(self.pa.encode('chief'), 'CHAYEHf')
        self.assertEqual(self.pa.encode('Japanese'), 'jAEpAEnIYz')
        self.assertEqual(self.pa.encode('stream'), 'strIYm')
        self.assertEqual(self.pa.encode('plants'), 'plAEntz')
        self.assertEqual(self.pa.encode('rhythm'), 'rIHTHm')
        self.assertEqual(self.pa.encode('eight'), 'EYt')
        self.assertEqual(self.pa.encode('science'), 'sAYEHns')
        self.assertEqual(self.pa.encode('major'), 'mAEjER')
        self.assertEqual(self.pa.encode('observe'), 'AAbsERv')
        self.assertEqual(self.pa.encode('tube'), 'tUWb')
        self.assertEqual(self.pa.encode('necessary'), 'nIYsEHsAArIH')
        self.assertEqual(self.pa.encode('weight'), 'wEYt')
        self.assertEqual(self.pa.encode('meat'), 'mIYt')
        self.assertEqual(self.pa.encode('lifted'), 'lIHftIHd')
        self.assertEqual(self.pa.encode('process'), 'prOWsEHs')
        self.assertEqual(self.pa.encode('army'), 'AArmIY')
        self.assertEqual(self.pa.encode('hat'), 'hAEt')
        self.assertEqual(self.pa.encode('property'), 'prOWpERtIH')
        self.assertEqual(self.pa.encode('particular'), 'pAArtIHkyUWlER')
        self.assertEqual(self.pa.encode('swim'), 'swIHm')
        self.assertEqual(self.pa.encode('terms'), 'tERmz')
        self.assertEqual(self.pa.encode('current'), 'kERrEHnt')
        self.assertEqual(self.pa.encode('park'), 'pAArk')
        self.assertEqual(self.pa.encode('sell'), 'sEHl')
        self.assertEqual(self.pa.encode('shoulder'), 'SHUHdER')
        self.assertEqual(self.pa.encode('industry'), 'IHndAHstrIH')
        self.assertEqual(self.pa.encode('wash'), 'wAASH')
        self.assertEqual(self.pa.encode('block'), 'blAAk')
        self.assertEqual(self.pa.encode('spread'), 'sprEHd')
        self.assertEqual(self.pa.encode('cattle'), 'kAEttl')
        self.assertEqual(self.pa.encode('wife'), 'wAYf')
        self.assertEqual(self.pa.encode('sharp'), 'SHAArp')
        self.assertEqual(self.pa.encode('company'), 'kAAmpAEnIH')
        self.assertEqual(self.pa.encode('radio'), 'rEYdIHOW')
        self.assertEqual(self.pa.encode("we'll"), 'wEHl')
        self.assertEqual(self.pa.encode('action'), 'AEkSHAXn')
        self.assertEqual(self.pa.encode('capital'), 'kAEpIHtAXl')
        self.assertEqual(self.pa.encode('factories'), 'fAEktAOrIYs')
        self.assertEqual(self.pa.encode('settled'), 'sEHttld')
        self.assertEqual(self.pa.encode('yellow'), 'yEHlOW')
        self.assertEqual(self.pa.encode("isn't"), 'IHzAXnt')
        self.assertEqual(self.pa.encode('southern'), 'sAWDHERn')
        self.assertEqual(self.pa.encode('truck'), 'trAHk')
        self.assertEqual(self.pa.encode('train'), 'trEYn')
        self.assertEqual(self.pa.encode('printed'), 'prIHntIHd')
        self.assertEqual(self.pa.encode("wouldn't"), 'wUHdnt')
        self.assertEqual(self.pa.encode('ahead'), 'EYhEHd')
        self.assertEqual(self.pa.encode('chance'), 'CHAEns')
        self.assertEqual(self.pa.encode('born'), 'bAOrn')
        self.assertEqual(self.pa.encode('level'), 'lIYvEHl')
        self.assertEqual(self.pa.encode('triangle'), 'trIHAENGgAXl')
        self.assertEqual(self.pa.encode('molecules'), 'mOWlEHkyUWlz')
        self.assertEqual(self.pa.encode('France'), 'frAEns')
        self.assertEqual(self.pa.encode('repeated'), 'rIYpIYtIHd')
        self.assertEqual(self.pa.encode('column'), 'kAAlAHmn')
        self.assertEqual(self.pa.encode('western'), 'wEHstERn')
        self.assertEqual(self.pa.encode('church'), 'CHERCH')
        self.assertEqual(self.pa.encode('sister'), 'sIHstER')
        self.assertEqual(self.pa.encode('oxygen'), 'AAksIHjEHn')
        self.assertEqual(self.pa.encode('plural'), 'plUHrAXl')
        self.assertEqual(self.pa.encode('various'), 'vEHrIHAXs')
        self.assertEqual(self.pa.encode('agreed'), 'AEgrIYd')
        self.assertEqual(self.pa.encode('opposite'), 'AAppAAzAYt')
        self.assertEqual(self.pa.encode('wrong'), 'rAONG')
        self.assertEqual(self.pa.encode('chart'), 'CHAArt')
        self.assertEqual(self.pa.encode('prepared'), 'prEHpEHrd')
        self.assertEqual(self.pa.encode('pretty'), 'prEHttIH')
        self.assertEqual(self.pa.encode('solution'), 'sAAlUWSHAXn')
        self.assertEqual(self.pa.encode('fresh'), 'frEHSH')
        self.assertEqual(self.pa.encode('shop'), 'SHAAp')
        self.assertEqual(self.pa.encode('suffix'), 'sAHffIHks')
        self.assertEqual(self.pa.encode('especially'), 'EHspEHSHAXlIY')
        self.assertEqual(self.pa.encode('shoes'), 'SHOWz')
        self.assertEqual(self.pa.encode('actually'), 'AEkCHUWAXlIY')
        self.assertEqual(self.pa.encode('nose'), 'nOWz')
        self.assertEqual(self.pa.encode('afraid'), 'AEfrEYd')
        self.assertEqual(self.pa.encode('dead'), 'dEHd')
        self.assertEqual(self.pa.encode('sugar'), 'sUWgER')
        self.assertEqual(self.pa.encode('adjective'), 'AEdjEHktAYv')
        self.assertEqual(self.pa.encode('fig'), 'fIHg')
        self.assertEqual(self.pa.encode('office'), 'AOffIHs')
        self.assertEqual(self.pa.encode('huge'), 'hyUWj')
        self.assertEqual(self.pa.encode('gun'), 'gAHn')
        self.assertEqual(self.pa.encode('similar'), 'sIHmIHlER')
        self.assertEqual(self.pa.encode('death'), 'dIYTH')
        self.assertEqual(self.pa.encode('score'), 'skAOr')
        self.assertEqual(self.pa.encode('forward'), 'fAOrwAOrd')
        self.assertEqual(self.pa.encode('stretched'), 'strEHtCHd')
        self.assertEqual(self.pa.encode('experience'), 'EHkspIYrIYEHns')
        self.assertEqual(self.pa.encode('rose'), 'rOWz')
        self.assertEqual(self.pa.encode('allow'), 'AOlOW')
        self.assertEqual(self.pa.encode('fear'), 'fIYr')
        self.assertEqual(self.pa.encode('workers'), 'wERkERs')
        self.assertEqual(self.pa.encode('Washington'), 'wAASHIHNGtAXn')
        self.assertEqual(self.pa.encode('Greek'), 'grIYk')
        self.assertEqual(self.pa.encode('women'), 'wOWmEHn')
        self.assertEqual(self.pa.encode('brought'), 'brAOt')
        self.assertEqual(self.pa.encode('led'), 'lEHd')
        self.assertEqual(self.pa.encode('march'), 'mAArCH')
        self.assertEqual(self.pa.encode('northern'), 'nAOrDHERn')
        self.assertEqual(self.pa.encode('create'), 'krIYt')
        self.assertEqual(self.pa.encode('British'), 'brAYtIHSH')
        self.assertEqual(self.pa.encode('difficult'), 'dIHffIHkAHlt')
        self.assertEqual(self.pa.encode('match'), 'mAEtCH')
        self.assertEqual(self.pa.encode('win'), 'wIHn')
        self.assertEqual(self.pa.encode("doesn't"), 'dAHznt')
        self.assertEqual(self.pa.encode('steel'), 'stIYl')
        self.assertEqual(self.pa.encode('total'), 'tAAtAXl')
        self.assertEqual(self.pa.encode('deal'), 'dIYl')
        self.assertEqual(self.pa.encode('determine'), 'dIHtERmAYn')
        self.assertEqual(self.pa.encode('evening'), 'IYvIYnIHNG')
        self.assertEqual(self.pa.encode('nor'), 'nAOr')
        self.assertEqual(self.pa.encode('rope'), 'rOWp')
        self.assertEqual(self.pa.encode('cotton'), 'kAAttAXn')
        self.assertEqual(self.pa.encode('apple'), 'AEppAXl')
        self.assertEqual(self.pa.encode('details'), 'dIHtEYlz')
        self.assertEqual(self.pa.encode('entire'), 'EHntAYr')
        self.assertEqual(self.pa.encode('corn'), 'kAOrn')
        self.assertEqual(self.pa.encode('substances'), 'sAHbstAEnsIHz')
        self.assertEqual(self.pa.encode('smell'), 'smEHl')
        self.assertEqual(self.pa.encode('tools'), 'tUWlz')
        self.assertEqual(self.pa.encode('conditions'), 'kAAndIHSHAXnz')
        self.assertEqual(self.pa.encode('cows'), 'kOWz')
        self.assertEqual(self.pa.encode('track'), 'trAEk')
        self.assertEqual(self.pa.encode('arrived'), 'AXrIHvd')
        self.assertEqual(self.pa.encode('located'), 'lOWkEYtIHd')
        self.assertEqual(self.pa.encode('sir'), 'sER')
        self.assertEqual(self.pa.encode('seat'), 'sIYt')
        self.assertEqual(self.pa.encode('division'), 'dIHvIHZHAXn')
        self.assertEqual(self.pa.encode('effect'), 'EHffEHkt')
        self.assertEqual(self.pa.encode('underline'), 'AHndERlAYn')
        self.assertEqual(self.pa.encode('view'), 'vyUW')

        # non-English words (with letters not used in English)
        self.assertEqual(self.pa.encode('garçon'), 'gAArÇAAn')
        self.assertEqual(self.pa.encode('ðæt'), 'ÐÆt')
        self.assertEqual(self.pa.encode('wünschen'), 'wÜnsCHEHn')
        self.assertEqual(self.pa.encode('øl'), 'Øl')


if __name__ == '__main__':
    unittest.main()
